import os
import sys
import unittest

from io import StringIO

from nose.config import Config
from nose.core import TestProgram
from nose.plugins.manager import PluginManager

support = os.path.join(os.path.dirname(__file__), 'support')


class TestResultSummary(unittest.TestCase):

    def test_with_todo_plugin(self):
        pkpath = os.path.join(support, 'todo')
        sys.path.insert(0, pkpath)
        from todoplug import TodoPlugin

        stream = StringIO()
        config = Config(stream=stream,
                        plugins=PluginManager([TodoPlugin()]))

        TestProgram(argv=['t', '--with-todo', pkpath],
                    config=config, exit=False)
        out = stream.getvalue()
        print
        out
        self.assert_('FAILED (TODO=1)' in out)


if __name__ == '__main__':
    unittest.main()



import os
import sys
import unittest

import nose.config
import nose.importer


class TestImporter(unittest.TestCase):

    def setUp(self):
        self.p = sys.path[:]

    def tearDown(self):
        sys.path = self.p[:]

    def test_add_paths(self):
        where = os.path.abspath(os.path.join(os.path.dirname(__file__),
                                             'support'))
        foo = os.path.join(where, 'foo')
        foobar = os.path.join(foo, 'bar')
        nose.importer.add_path(foobar)

        assert not foobar in sys.path
        assert not foo in sys.path
        assert where in sys.path
        assert sys.path[0] == where, "%s first should be %s" % (sys.path, where)

    def test_import(self):
        where = os.path.abspath(os.path.join(os.path.dirname(__file__),
                                             'support'))
        foo = os.path.join(where, 'foo')
        foobar = os.path.join(foo, 'bar')

        imp = nose.importer.Importer()
        mod = imp.importFromDir(foobar, 'buz')
        assert where in sys.path
        # buz has an intra-package import that sets boodle
        assert mod.boodle

    def test_module_no_file(self):
        where = os.path.abspath(os.path.join(os.path.dirname(__file__),
                                             'support'))
        foo = os.path.join(where, 'foo')
        foobar = os.path.join(foo, 'bar')

        # something that's not a real module and has no __file__
        sys.modules['buz'] = 'Whatever'

        imp = nose.importer.Importer()
        mod = imp.importFromDir(foobar, 'buz')
        assert where in sys.path
        # buz has an intra-package import that sets boodle
        assert mod.boodle

    def test_module_init_prefix(self):
        where = os.path.abspath(os.path.join(os.path.dirname(__file__),
                                             'support', 'init_prefix_bug'))
        nose.importer.add_path(where)
        mod = os.path.join(where, '__init__not.py')
        fqname = 'init_prefix_bug.__init__not'

        imp = nose.importer.Importer()
        mod = imp.importFromPath(mod, fqname)


if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
import sys
import unittest

from nose.exc import SkipTest


class TestForXunit(unittest.TestCase):

    def test_success(self):
        pass

    def test_fail(self):
        self.assertEqual("this", "that")

    def test_error(self):
        raise TypeError("oops, wrong type")

    def test_non_ascii_error(self):
        raise Exception(u"日本")

    def test_output(self):
        sys.stdout.write("test-generated output\n")

    def test_skip(self):
        raise SkipTest("skipit")



"""
Writing Plugins
---------------

nose supports plugins for test collection, selection, observation and
reporting. There are two basic rules for plugins:

* Plugin classes should subclass :class:`nose.plugins.Plugin`.

* Plugins may implement any of the methods described in the class
  :doc:`IPluginInterface <interface>` in nose.plugins.base. Please note that
  this class is for documentary purposes only; plugins may not subclass
  IPluginInterface.

Hello World
===========

Here's a basic plugin.  It doesn't do much so read on for more ideas or dive
into the :doc:`IPluginInterface <interface>` to see all available hooks.

.. code-block:: python

    import logging
    import os

    from nose.plugins import Plugin

    log = logging.getLogger('nose.plugins.helloworld')

    class HelloWorld(Plugin):
        name = 'helloworld'

        def options(self, parser, env=os.environ):
            super(HelloWorld, self).options(parser, env=env)

        def configure(self, options, conf):
            super(HelloWorld, self).configure(options, conf)
            if not self.enabled:
                return

        def finalize(self, result):
            log.info('Hello pluginized world!')

Registering
===========

.. Note::
  Important note: the following applies only to the default
  plugin manager. Other plugin managers may use different means to
  locate and load plugins.

For nose to find a plugin, it must be part of a package that uses
setuptools_, and the plugin must be included in the entry points defined
in the setup.py for the package:

.. code-block:: python

    setup(name='Some plugin',
        # ...
        entry_points = {
            'nose.plugins.0.10': [
                'someplugin = someplugin:SomePlugin'
                ]
            },
        # ...
        )

Once the package is installed with install or develop, nose will be able
to load the plugin.

.. _setuptools: http://peak.telecommunity.com/DevCenter/setuptools

Registering a plugin without setuptools
=======================================

It is currently possible to register a plugin programmatically by
creating a custom nose runner like this :

.. code-block:: python

    import nose
    from yourplugin import YourPlugin

    if __name__ == '__main__':
        nose.main(addplugins=[YourPlugin()])

Defining options
================

All plugins must implement the methods ``options(self, parser, env)``
and ``configure(self, options, conf)``. Subclasses of nose.plugins.Plugin
that want the standard options should call the superclass methods.

nose uses optparse.OptionParser from the standard library to parse
arguments. A plugin's ``options()`` method receives a parser
instance. It's good form for a plugin to use that instance only to add
additional arguments that take only long arguments (--like-this). Most
of nose's built-in arguments get their default value from an environment
variable.

A plugin's ``configure()`` method receives the parsed ``OptionParser`` options
object, as well as the current config object. Plugins should configure their
behavior based on the user-selected settings, and may raise exceptions
if the configured behavior is nonsensical.

Logging
=======

nose uses the logging classes from the standard library. To enable users
to view debug messages easily, plugins should use ``logging.getLogger()`` to
acquire a logger in the ``nose.plugins`` namespace.

Recipes
=======

* Writing a plugin that monitors or controls test result output

  Implement any or all of ``addError``, ``addFailure``, etc., to monitor test
  results. If you also want to monitor output, implement
  ``setOutputStream`` and keep a reference to the output stream. If you
  want to prevent the builtin ``TextTestResult`` output, implement
  ``setOutputSteam`` and *return a dummy stream*. The default output will go
  to the dummy stream, while you send your desired output to the real stream.

  Example: `examples/html_plugin/htmlplug.py`_

* Writing a plugin that handles exceptions

  Subclass :doc:`ErrorClassPlugin <errorclasses>`.

  Examples: :doc:`nose.plugins.deprecated <deprecated>`,
  :doc:`nose.plugins.skip <skip>`

* Writing a plugin that adds detail to error reports

  Implement ``formatError`` and/or ``formatFailure``. The error tuple
  you return (error class, error message, traceback) will replace the
  original error tuple.

  Examples: :doc:`nose.plugins.capture <capture>`,
  :doc:`nose.plugins.failuredetail <failuredetail>`

* Writing a plugin that loads tests from files other than python modules

  Implement ``wantFile`` and ``loadTestsFromFile``. In ``wantFile``,
  return True for files that you want to examine for tests. In
  ``loadTestsFromFile``, for those files, return an iterable
  containing TestCases (or yield them as you find them;
  ``loadTestsFromFile`` may also be a generator).

  Example: :doc:`nose.plugins.doctests <doctests>`

* Writing a plugin that prints a report

  Implement ``begin`` if you need to perform setup before testing
  begins. Implement ``report`` and output your report to the provided stream.

  Examples: :doc:`nose.plugins.cover <cover>`, :doc:`nose.plugins.prof <prof>`

* Writing a plugin that selects or rejects tests

  Implement any or all ``want*``  methods. Return False to reject the test
  candidate, True to accept it -- which  means that the test candidate
  will pass through the rest of the system, so you must be prepared to
  load tests from it if tests can't be loaded by the core loader or
  another plugin -- and None if you don't care.

  Examples: :doc:`nose.plugins.attrib <attrib>`,
  :doc:`nose.plugins.doctests <doctests>`, :doc:`nose.plugins.testid <testid>`


More Examples
=============

See any builtin plugin or example plugin in the examples_ directory in
the nose source distribution. There is a list of third-party plugins
`on jottit`_.

.. _examples/html_plugin/htmlplug.py: http://python-nose.googlecode.com/svn/trunk/examples/html_plugin/htmlplug.py
.. _examples: http://python-nose.googlecode.com/svn/trunk/examples
.. _on jottit: http://nose-plugins.jottit.com/

"""
from nose.plugins.base import Plugin
from nose.plugins.manager import *
from nose.plugins.plugintest import PluginTester

if __name__ == '__main__':
    import doctest

    doctest.testmod()



import unittest
from optparse import OptionParser

from io import StringIO

from nose.config import Config
from nose.plugins.deprecated import Deprecated, DeprecatedTest
from nose.result import _TextTestResult

try:
    # 2.7+
    from unittest.runner import _WritelnDecorator
except ImportError:
    from unittest import _WritelnDecorator


class TestDeprecatedPlugin(unittest.TestCase):

    def test_api_present(self):
        sk = Deprecated()
        sk.addOptions
        sk.configure
        sk.prepareTestResult

    def test_prepare_patches_result(self):
        stream = _WritelnDecorator(StringIO())
        res = _TextTestResult(stream, 0, 1)
        sk = Deprecated()
        sk.prepareTestResult(res)
        res._orig_addError
        res._orig_printErrors
        res._orig_wasSuccessful
        res.deprecated
        self.assertEqual(
            res.errorClasses,
            {DeprecatedTest: (res.deprecated, 'DEPRECATED', False)})

        # result w/out print works too
        res = unittest.TestResult()
        sk = Deprecated()
        sk.prepareTestResult(res)
        res._orig_addError
        res.deprecated
        self.assertEqual(
            res.errorClasses,
            {DeprecatedTest: (res.deprecated, 'DEPRECATED', False)})

    def test_patched_result_handles_deprecated(self):
        res = unittest.TestResult()
        sk = Deprecated()
        sk.prepareTestResult(res)

        class TC(unittest.TestCase):
            def test(self):
                raise DeprecatedTest('deprecated me')

        test = TC('test')
        test(res)
        assert not res.errors, "Deprecated was not caught: %s" % res.errors
        assert res.deprecated
        assert res.deprecated[0][0] is test

    def test_patches_only_when_needed(self):
        class NoPatch(unittest.TestResult):
            def __init__(self):
                self.errorClasses = {}

        res = NoPatch()
        sk = Deprecated()
        sk.prepareTestResult(res)
        assert not hasattr(res, '_orig_addError'), \
            "Deprecated patched a result class it didn't need to patch"

    def test_deprecated_output(self):
        class TC(unittest.TestCase):
            def test(self):
                raise DeprecatedTest('deprecated me')

        stream = _WritelnDecorator(StringIO())
        res = _TextTestResult(stream, 0, 1)
        sk = Deprecated()
        sk.prepareTestResult(res)

        test = TC('test')
        test(res)
        assert not res.errors, "Deprecated was not caught: %s" % res.errors
        assert res.deprecated

        res.printErrors()
        out = stream.getvalue()
        assert out
        assert out.strip() == "D"
        assert res.wasSuccessful()

    def test_deprecated_output_verbose(self):
        class TC(unittest.TestCase):
            def test(self):
                raise DeprecatedTest('deprecated me too')

        stream = _WritelnDecorator(StringIO())
        res = _TextTestResult(stream, 0, verbosity=2)
        sk = Deprecated()
        sk.prepareTestResult(res)
        test = TC('test')
        test(res)
        assert not res.errors, "Deprecated was not caught: %s" % res.errors
        assert res.deprecated

        res.printErrors()
        out = stream.getvalue()
        print
        out
        assert out

        assert ' ... DEPRECATED' in out
        assert 'deprecated me too' in out

    def test_enabled_by_default(self):
        sk = Deprecated()
        assert sk.enabled, "Deprecated was not enabled by default"

    def test_can_be_disabled(self):
        parser = OptionParser()
        sk = Deprecated()
        sk.addOptions(parser)
        options, args = parser.parse_args(['--no-deprecated'])
        sk.configure(options, Config())
        assert not sk.enabled, \
            "Deprecated was not disabled by noDeprecated option"


if __name__ == '__main__':
    unittest.main()



import maths
from test_pak import state


def setup():
    state.append('test_pak.test_sub.test_mod.setup')


def test():
    state.append('test_pak.test_sub.test_mod.test')
    assert maths.add(1, 2) == 3


class TestMaths:

    def setup_class(cls):
        state.append('test_pak.test_sub.test_mod.TestMaths.setup_class')

    setup_class = classmethod(setup_class)

    def teardown_class(cls):
        state.append('test_pak.test_sub.test_mod.TestMaths.teardown_class')

    teardown_class = classmethod(teardown_class)

    def setup(self):
        state.append('test_pak.test_sub.test_mod.TestMaths.setup')

    def teardown(self):
        state.append('test_pak.test_sub.test_mod.TestMaths.teardown')

    def test_div(self):
        state.append('test_pak.test_sub.test_mod.TestMaths.test_div')
        assert maths.div(2, 1) == 2, "%s != %s" % (maths.div(2, 1), 2)

    def test_two_two(self):
        state.append('test_pak.test_sub.test_mod.TestMaths.test_two_two')
        assert maths.mult(2, 2) == maths.add(2, 2)


def teardown():
    state.append('test_pak.test_sub.test_mod.teardown')



def test_evens():
    yield check_even_cls


class Test(object):
    def test_evens(self):
        yield check_even_cls


class Check(object):
    def __call__(self):
        pass


check_even_cls = Check()



import imp
import sys
import unittest

from mock import ResultProxyFactory, ResultProxy

from nose import case
from nose.config import Config
from nose.suite import LazySuite, ContextSuite, ContextSuiteFactory, \
    ContextList


class TestLazySuite(unittest.TestCase):

    def setUp(self):
        class TC(unittest.TestCase):
            def test_one(self):
                pass

            def test_two(self):
                pass

        self.TC = TC

    def test_test_generator(self):
        TC = self.TC
        tests = [TC('test_one'), TC('test_two')]

        def gen_tests():
            for test in tests:
                yield test

        suite = LazySuite(gen_tests)
        self.assertEqual(list([test for test in suite]), tests)

    def test_lazy_and_nonlazy(self):
        TC = self.TC
        tests = [TC('test_one'), TC('test_two')]

        def gen_tests():
            for test in tests:
                yield test

        nonlazy = LazySuite(tests)
        lazy = LazySuite(gen_tests)

        assert lazy
        assert nonlazy

        lazytests = []
        nonlazytests = []
        for t in lazy:
            print("lazy %s" % t)
            lazytests.append(t)
        for t in nonlazy:
            print("nonlazy %s" % t)
            nonlazytests.append(t)
        slazy = map(str, lazytests)
        snonlazy = map(str, nonlazytests)
        assert slazy == snonlazy, \
            "Lazy and Nonlazy produced different test lists (%s vs %s)" \
            % (slazy, snonlazy)

    def test_lazy_nonzero(self):
        """__nonzero__ works correctly for lazy suites"""

        TC = self.TC
        tests = [TC('test_one'), TC('test_two')]

        def gen_tests():
            for test in tests:
                yield test

        lazy = LazySuite(gen_tests)
        assert lazy
        assert lazy
        assert lazy

        count = 0
        for test in lazy:
            print(test)
            assert test
            count += 1
        self.assertEqual(count, 2, "Expected 2 tests, got %s" % count)
        assert lazy

        def gen_tests_empty():
            for test in []:
                yield test
            return

        empty = LazySuite(gen_tests_empty)
        assert not empty
        for test in empty:
            assert False, "Loaded a test from empty suite: %s" % test


class TestContextSuite(unittest.TestCase):

    def setUp(self):
        class TC(unittest.TestCase):
            def test_one(self):
                pass

            def test_two(self):
                pass

        self.TC = TC

    def test_tests_are_wrapped(self):
        """Tests in a context suite are wrapped"""
        suite = ContextSuite(
            [self.TC('test_one'), self.TC('test_two')])
        for test in suite:
            assert isinstance(test.test, self.TC)

    def test_nested_context_suites(self):
        """Nested suites don't re-wrap"""
        suite = ContextSuite(
            [self.TC('test_one'), self.TC('test_two')])
        suite2 = ContextSuite(suite)
        suite3 = ContextSuite([suite2])

        # suite3 is [suite2]
        tests = [t for t in suite3]
        assert isinstance(tests[0], ContextSuite)
        # suite2 is [suite]
        tests = [t for t in tests[0]]
        assert isinstance(tests[0], ContextSuite)
        # suite is full of wrapped tests
        tests = [t for t in tests[0]]
        cases = filter(lambda t: isinstance(t, case.Test), tests)
        assert cases
        assert len(cases) == len(tests)

        # sub-suites knows they have a context
        # assert suite.context is None
        # assert suite2.context is suite
        # assert suite3.context is suite2

    def test_context_fixtures_called(self):
        class P:
            was_setup = False
            was_torndown = False

            def setup(self):
                self.was_setup = True

            def teardown(self):
                self.was_torndown = True

        context = P()
        suite = ContextSuite(
            [self.TC('test_one'), self.TC('test_two')],
            context=context)
        res = unittest.TestResult()
        suite(res)

        assert not res.errors, res.errors
        assert not res.failures, res.failures
        assert context.was_setup
        assert context.was_torndown

    def test_context_fixtures_for_ancestors(self):
        top = imp.new_module('top')
        top.bot = imp.new_module('top.bot')
        top.bot.end = imp.new_module('top.bot.end')

        sys.modules['top'] = top
        sys.modules['top.bot'] = top.bot
        sys.modules['top.bot.end'] = top.bot.end

        class TC(unittest.TestCase):
            def runTest(self):
                pass

        top.bot.TC = TC
        TC.__module__ = 'top.bot'

        # suite with just TC test
        # this suite should call top and top.bot setup
        csf = ContextSuiteFactory()
        suite = csf(ContextList([TC()], context=top.bot))

        suite.setUp()
        assert top in csf.was_setup, "Ancestor not set up"
        assert top.bot in csf.was_setup, "Context not set up"
        suite.has_run = True
        suite.tearDown()
        assert top in csf.was_torndown, "Ancestor not torn down"
        assert top.bot in csf.was_torndown, "Context not torn down"

        # wrapped suites
        # the outer suite sets up its context, the inner
        # its context only, without re-setting up the outer context
        csf = ContextSuiteFactory()
        inner_suite = csf(ContextList([TC()], context=top.bot))
        suite = csf(ContextList(inner_suite, context=top))

        suite.setUp()
        assert top in csf.was_setup
        assert not top.bot in csf.was_setup
        inner_suite.setUp()
        assert top in csf.was_setup
        assert top.bot in csf.was_setup
        assert csf.was_setup[top] is suite
        assert csf.was_setup[top.bot] is inner_suite

    def test_context_fixtures_setup_fails(self):
        class P:
            was_setup = False
            was_torndown = False

            def setup(self):
                self.was_setup = True
                assert False, "Setup failed"

            def teardown(self):
                self.was_torndown = True

        context = P()
        suite = ContextSuite(
            [self.TC('test_one'), self.TC('test_two')],
            context=context)
        res = unittest.TestResult()
        suite(res)

        assert not res.failures, res.failures
        assert res.errors, res.errors
        assert context.was_setup
        assert not context.was_torndown
        assert res.testsRun == 0, \
            "Expected to run no tests but ran %s" % res.testsRun

    def test_context_fixtures_no_tests_no_setup(self):
        class P:
            was_setup = False
            was_torndown = False

            def setup(self):
                self.was_setup = True

            def teardown(self):
                self.was_torndown = True

        context = P()
        suite = ContextSuite([], context=context)
        res = unittest.TestResult()
        suite(res)

        assert not res.failures, res.failures
        assert not res.errors, res.errors
        assert not context.was_setup
        assert not context.was_torndown
        assert res.testsRun == 0, \
            "Expected to run no tests but ran %s" % res.testsRun

    def test_result_proxy_used(self):
        class TC(unittest.TestCase):
            def runTest(self):
                raise Exception("error")

        ResultProxy.called[:] = []
        res = unittest.TestResult()
        config = Config()

        suite = ContextSuite([TC()], resultProxy=ResultProxyFactory())
        suite(res)
        calls = [c[0] for c in ResultProxy.called]
        self.assertEqual(calls, ['beforeTest', 'startTest',
                                 'addError', 'stopTest', 'afterTest'])


class TestContextSuiteFactory(unittest.TestCase):

    def test_ancestry(self):
        top = imp.new_module('top')
        top.bot = imp.new_module('top.bot')
        top.bot.end = imp.new_module('top.bot.end')

        sys.modules['top'] = top
        sys.modules['top.bot'] = top.bot
        sys.modules['top.bot.end'] = top.bot.end

        class P:
            pass

        top.bot.P = P
        P.__module__ = 'top.bot'

        csf = ContextSuiteFactory()
        P_ancestors = list([a for a in csf.ancestry(P)])
        self.assertEqual(P_ancestors, [top.bot, top])

        end_ancestors = list([a for a in csf.ancestry(top.bot.end)])
        self.assertEqual(end_ancestors, [top.bot, top])

        bot_ancestors = list([a for a in csf.ancestry(top.bot)])
        self.assertEqual(bot_ancestors, [top])

        top_ancestors = list([a for a in csf.ancestry(top)])
        self.assertEqual(top_ancestors, [])


if __name__ == '__main__':
    import logging

    logging.basicConfig(level=logging.DEBUG)
    unittest.main()

#     class TC(unittest.TestCase):
#             def runTest(self):
#                 raise Exception("error")

#     ResultProxy.called[:] = []
#     res = unittest.TestResult()
#     config = Config()



#!/usr/bin/env python

"""Test the copy of nose in this directory, by running that nose against itself.

You can test nose using nose in other ways, but if you don't use this script,
you might have one installation of nose testing another installation, which is
not supported.
"""

# More detail:

# In the absence of some sort of deep renaming magic, nose can't reasonably
# test a different installation of itself, given the existence of the global
# module registry sys.modules .

# If installed system-wide with setuptools, setuptools (via the site-packages
# easy-install.pth) takes you at your word and ensures that the installed nose
# comes first on sys.path .  So the only way to test a copy of nose other than
# the installed one is to install that version (e.g. by running python setup.py
# develop).

# This script provides a way of running nose on nose's own tests without
# installing the version to be tested, nor uninstalling the currently-installed
# version.

import glob
import os
import sys

import pkg_resources

if __name__ == "__main__":
    this_dir = os.path.normpath(os.path.abspath(os.path.dirname(__file__)))
    lib_dirs = [this_dir]
    test_dir = this_dir
    if sys.version_info >= (3,):
        # Under Python 3.x, we need to 'build' the source (using 2to3, etc)
        # first.  'python3 setup.py build_tests' will put everything under
        # build/tests (including nose itself, since some tests are inside the
        # nose source)
        # The 'py3where' argument in setup.cfg will take care of making sure we
        # pull our tests only from the build/tests directory.  We just need to
        # make sure the right things are on sys.path.
        lib_dirs = glob.glob(os.path.join(this_dir, 'build', 'lib*'))
        test_dir = os.path.join(this_dir, 'build', 'tests')
        if not os.path.isdir(test_dir):
            raise Exception(
                "Error: %s does not exist.  Use the setup.py 'build_tests' command to create it." % (
                    test_dir,))

    env = pkg_resources.Environment(search_path=lib_dirs)

    distributions = env["nose-py3"]
    if not distributions:
        raise Exception("No nose distibution has been built")
    elif len(distributions) != 1:
        raise Exception("More than one nose distribution found")

    dist = distributions[0]
    dist.activate()

    # Always make sure our chosen test dir is first on the path
    sys.path.insert(0, test_dir)
    import nose

    nose.run_exit()



from nose.plugins.errorclass import ErrorClass, ErrorClassPlugin


class Todo(Exception):
    pass


class TodoPlugin(ErrorClassPlugin):
    name = "todo"

    todo = ErrorClass(Todo, label='TODO', isfailure=True)


class NonFailureTodoPlugin(ErrorClassPlugin):
    name = "non-failure-todo"

    todo = ErrorClass(Todo, label='TODO', isfailure=False)



import sys

called = []

_multiprocess_can_split_ = 1


def setup():
    print >> sys.stderr, "setup called"
    called.append('setup')


def teardown():
    print >> sys.stderr, "teardown called"
    called.append('teardown')


def test_a():
    assert len(called) == 1, "len(%s) !=1" % called


def test_b():
    assert len(called) == 1, "len(%s) !=1" % called


class TestMe:
    def setup_class(cls):
        cls._setup = True

    setup_class = classmethod(setup_class)

    def test_one(self):
        assert self._setup, "Class was not set up"



import os
from optparse import OptionParser

from nose.config import Config
from nose.plugins.cover import Coverage
from nose.tools import eq_


class TestCoveragePlugin(object):

    def test_cover_options_packages(self):
        _test_options_helper('--cover-package', 'coverPackages',
                             ['pkg1', 'pkg2', 'pkg3'], [],
                             'pkg1,pkg2,pkg3', 'NOSE_COVER_PACKAGE')

    def test_cover_options_noprint(self):
        _test_options_helper('--cover-no-print', 'coverPrint',
                             False, True,
                             env_key='NOSE_COVER_NO_PRINT')

    def test_cover_options_erase(self):
        _test_options_helper('--cover-erase', 'coverErase',
                             True, False,
                             env_key='NOSE_COVER_ERASE')

    def test_cover_options_tests(self):
        _test_options_helper('--cover-tests', 'coverTests',
                             True, False,
                             env_key='NOSE_COVER_TESTS')

    def test_cover_options_config_file(self):
        def get_sys_info(cov_inst):
            return cov_inst.sys_info()

        def get_config_files(cov_info):
            cov_info = dict(cov_info)
            if 'config_files' in cov_info:
                return cov_info['config_files']
            return None

        f = open('not_default_config_file', 'wb')
        f.close()
        try:
            c1, c2 = _test_options_helper(
                '--cover-config-file', 'coverConfigFile',
                'not_default_config_file', True,
                arg_value='not_default_config_file',
                env_key='NOSE_COVER_CONFIG_FILE')

            cov_info = get_sys_info(c1.coverInstance)
            config_files = get_config_files(cov_info)
            if config_files is not None:
                assert '.coveragerc' in config_files
            else:
                assert False, "coverage did not load default config file"

            cov_info = get_sys_info(c2.coverInstance)
            config_files = get_config_files(cov_info)
            if config_files is not None:
                assert 'not_default_config_file' in config_files
            else:
                assert False, "coverage did not load expected config file"
        finally:
            os.unlink('not_default_config_file')


def _test_options_helper(arg_option, cover_option,
                         expected_set, expected_not_set,
                         arg_value=None, env_key=None):
    prefix_args = ['test_can_be_disabled', '--with-coverage']

    # Assert that the default works as expected
    parser = OptionParser()
    c_arg_not_set = Coverage()
    c_arg_not_set.addOptions(parser)
    options, _ = parser.parse_args(prefix_args)
    c_arg_not_set.configure(options, Config())
    eq_(expected_not_set, getattr(c_arg_not_set, cover_option))

    # Assert that the argument parser picks up the expected value
    parser = OptionParser()
    c_arg_set = Coverage()
    c_arg_set.addOptions(parser)

    args = arg_option
    if arg_value is not None:
        args += '=' + arg_value

    options, _ = parser.parse_args(prefix_args + [args])
    c_arg_set.configure(options, Config())
    eq_(expected_set, getattr(c_arg_set, cover_option))

    # If the option supports environment variables, check that too
    if env_key is not None:
        args = 'true'
        if arg_value is not None:
            args = arg_value

        env = {env_key: args}
        c = Coverage()
        parser = OptionParser()
        c.addOptions(parser, env)
        options, _ = parser.parse_args(prefix_args)
        c.configure(options, Config())
        eq_(expected_set, getattr(c, cover_option))

    return c_arg_not_set, c_arg_set



import os
import signal
import sys
from subprocess import Popen, PIPE
from time import sleep

import nose
from nose.exc import SkipTest

support = os.path.join(os.path.dirname(__file__), 'support')

PYTHONPATH = os.environ['PYTHONPATH'] if 'PYTHONPATH' in os.environ else ''


def setup():
    nose_parent_dir = os.path.normpath(os.path.join(os.path.abspath(os.path.dirname(nose.__file__)), '..'))
    paths = [nose_parent_dir]
    if PYTHONPATH:
        paths.append(PYTHONPATH)
    os.environ['PYTHONPATH'] = os.pathsep.join(paths)


def teardown():
    if PYTHONPATH:
        os.environ['PYTHONPATH'] = PYTHONPATH
    else:
        del os.environ['PYTHONPATH']


def waitForKillFile(killfile):
    retry = 100
    while not os.path.exists(killfile):
        sleep(0.1)
        retry -= 1
        if not retry:
            raise Exception('Timeout while waiting for kill file to be created')
    os.remove(killfile)


runner = os.path.join(support, 'fake_nosetest.py')


def keyboardinterrupt(case):
    # os.setsid would create a process group so signals sent to the
    # parent process will propogates to all children processes
    if not hasattr(os, 'setsid') or not hasattr(os, 'killpg'):
        raise SkipTest("OS does not support os.setsid or os.killpg")

    from tempfile import mktemp
    logfile = mktemp()
    killfile = mktemp()
    process = Popen(
        [sys.executable, runner, os.path.join(support, case), logfile, killfile],
        preexec_fn=os.setsid, stdout=PIPE, stderr=PIPE, bufsize=-1)

    waitForKillFile(killfile)
    os.killpg(process.pid, signal.SIGINT)
    return process, logfile, killfile


def get_log_content(logfile):
    '''prefix = 'tempfile is: '
    if not stdout.startswith(prefix):
        raise Exception('stdout does not contain tmp file name: '+stdout)
    logfile = stdout[len(prefix):].strip() #remove trailing new line char'''
    f = open(logfile)
    content = f.read()
    f.close()
    os.remove(logfile)
    return content


def test_keyboardinterrupt():
    process, logfile, _ = keyboardinterrupt('keyboardinterrupt.py')
    stdout, stderr = [s.decode('utf-8') for s in process.communicate(None)]
    log = get_log_content(logfile)
    print
    stderr
    print
    '----'
    print
    stdout
    print
    '----'
    print
    log
    assert 'setup' in log
    assert 'test_timeout' in log
    assert 'test_timeout_finished' not in log
    assert 'test_pass' not in log
    assert 'teardown' in log
    assert 'Ran 0 tests' in stderr
    assert 'KeyboardInterrupt' in stderr
    assert 'FAILED (errors=1)' in stderr
    assert 'ERROR: Worker 0 keyboard interrupt, failing current test ' + os.path.join(support,
                                                                                      'keyboardinterrupt.py') in stderr


def test_keyboardinterrupt_twice():
    process, logfile, killfile = keyboardinterrupt('keyboardinterrupt_twice.py')
    waitForKillFile(killfile)
    os.killpg(process.pid, signal.SIGINT)
    stdout, stderr = [s.decode('utf-8') for s in process.communicate(None)]
    log = get_log_content(logfile)
    print
    stderr
    print
    '----'
    print
    stdout
    print
    '----'
    print
    log
    assert 'setup' in log
    assert 'test_timeout' in log
    assert 'test_timeout_finished' not in log
    assert 'test_pass' not in log
    assert 'teardown' in log
    assert 'teardown_finished' not in log
    assert 'Ran 0 tests' in stderr
    assert 'KeyboardInterrupt' in stderr
    assert 'FAILED (errors=1)' in stderr



"""Tools not exempt from being descended into in tracebacks"""

import time

__all__ = ['make_decorator', 'raises', 'set_trace', 'timed', 'with_setup',
           'TimeExpired', 'istest', 'nottest']


class TimeExpired(AssertionError):
    pass


def make_decorator(func):
    """
    Wraps a test decorator so as to properly replicate metadata
    of the decorated function, including nose's additional stuff
    (namely, setup and teardown).
    """

    def decorate(newfunc):
        if hasattr(func, 'compat_func_name'):
            name = func.compat_func_name
        else:
            name = func.__name__
        newfunc.__dict__ = func.__dict__
        newfunc.__doc__ = func.__doc__
        newfunc.__module__ = func.__module__
        if not hasattr(newfunc, 'compat_co_firstlineno'):
            newfunc.compat_co_firstlineno = func.func_code.co_firstlineno
        try:
            newfunc.__name__ = name
        except TypeError:
            # can't set func name in 2.3
            newfunc.compat_func_name = name
        return newfunc

    return decorate


def raises(*exceptions):
    """Test must raise one of expected exceptions to pass.

    Example use::

      @raises(TypeError, ValueError)
      def test_raises_type_error():
          raise TypeError("This test passes")

      @raises(Exception)
      def test_that_fails_by_passing():
          pass

    If you want to test many assertions about exceptions in a single test,
    you may want to use `assert_raises` instead.
    """
    valid = ' or '.join([e.__name__ for e in exceptions])

    def decorate(func):
        name = func.__name__

        def newfunc(*arg, **kw):
            try:
                func(*arg, **kw)
            except exceptions:
                pass
            except:
                raise
            else:
                message = "%s() did not raise %s" % (name, valid)
                raise AssertionError(message)

        newfunc = make_decorator(func)(newfunc)
        return newfunc

    return decorate


def set_trace():
    """Call pdb.set_trace in the calling frame, first restoring
    sys.stdout to the real output stream. Note that sys.stdout is NOT
    reset to whatever it was before the call once pdb is done!
    """
    import pdb
    import sys
    stdout = sys.stdout
    sys.stdout = sys.__stdout__
    pdb.Pdb().set_trace(sys._getframe().f_back)


def timed(limit):
    """Test must finish within specified time limit to pass.

    Example use::

      @timed(.1)
      def test_that_fails():
          time.sleep(.2)
    """

    def decorate(func):
        def newfunc(*arg, **kw):
            start = time.time()
            result = func(*arg, **kw)
            end = time.time()
            if end - start > limit:
                raise TimeExpired("Time limit (%s) exceeded" % limit)
            return result

        newfunc = make_decorator(func)(newfunc)
        return newfunc

    return decorate


def with_setup(setup=None, teardown=None):
    """Decorator to add setup and/or teardown methods to a test function::

      @with_setup(setup, teardown)
      def test_something():
          " ... "

    Note that `with_setup` is useful *only* for test functions, not for test
    methods or inside of TestCase subclasses.
    """

    def decorate(func, setup=setup, teardown=teardown):
        if setup:
            if hasattr(func, 'setup'):
                _old_s = func.setup

                def _s():
                    setup()
                    _old_s()

                func.setup = _s
            else:
                func.setup = setup
        if teardown:
            if hasattr(func, 'teardown'):
                _old_t = func.teardown

                def _t():
                    _old_t()
                    teardown()

                func.teardown = _t
            else:
                func.teardown = teardown
        return func

    return decorate


def istest(func):
    """Decorator to mark a function or method as a test
    """
    func.__test__ = True
    return func


def nottest(func):
    """Decorator to mark a function or method as *not* a test
    """
    func.__test__ = False
    return func



import os
import unittest

from nose.plugins import PluginTester

# from nose.plugins.builtin import FailureDetail, Capture, Doctest

support = os.path.join(os.path.dirname(__file__), 'support', 'issue408')


class TestIssue408(PluginTester, unittest.TestCase):
    args = ['--where=' + support, 'test:testa.test1', 'test:testa.test2', 'test:testb.test1',
            'test:testb.test2']
    activate = "-v"

    def makeSuite(self):
        # make PluginTester happy, because we don't specify suitepath, we
        # have to implement this function
        return None

    def test_no_failure(self):
        output = str(self.output)
        assert 'FAIL:' not in output
        assert 'AssertionError' not in output
        assert 'OK' in output


if __name__ == '__main__':
    unittest.main()



import unittest


def test_a():
    pass


def test_b():
    raise TypeError("I am typeless")


def test_c():
    assert False, "I am contrary"


def test_gen():
    def tryit(i):
        pass

    for i in range(0, 4):
        yield tryit, i


class TestCase(unittest.TestCase):
    def test_a(self):
        pass

    def test_b(self):
        pass


class TestCls:
    def test_a(self):
        pass

    def test_gen(self):
        def tryit(i):
            pass

        for i in range(0, 4):
            yield tryit, i

    def test_z(self):
        pass



from state import called


def setup():
    called.append('test_pak1.setup')


def teardown():
    called.append('test_pak1.teardown')


def test_one_one():
    called.append('test_pak1.test_one_one')


def test_one_two():
    called.append('test_pak1.test_one_two')



from test_pak import state


def setup():
    # print "SUB setup called", state, id(state)
    state.append('test_pak.test_sub.setup')


def teardown():
    # print "SUB teardown called", state, id(state)
    state.append('test_pak.test_sub.teardown')


def test_sub_init():
    state.append('test_pak.test_sub.test_sub_init')



""" Patch utility to apply unified diffs

    Brute-force line-by-line non-recursive parsing 

    Copyright (c) 2008-2010 anatoly techtonik
    Available under the terms of MIT license

    NOTE: This version has been patched by Alex Stewart <alex@foogod.com> for
    Python 3.x support and other misc fixups.

    Project home: http://code.google.com/p/python-patch/


    $Id: patch.py 92 2010-07-02 06:04:57Z techtonik $
    $HeadURL: http://python-patch.googlecode.com/svn/trunk/patch.py $
"""

__author__ = "techtonik.rainforce.org"
__version__ = "10.04-2.pAS1"

import copy
import logging
import re
import sys
from fileinput import filename
from logging import debug, info, warning
from io import StringIO
from os.path import isfile, abspath, exists
from os import unlink

_open = open

if sys.version_info >= (3,):
    # Open files with universal newline support but no newline translation (3.x)
    def open(filename, mode='r'):
        return _open(filename, mode, newline='')
else:
    # Open files with universal newline support but no newline translation (2.x)
    def open(filename, mode='r'):
        return _open(filename, mode + 'b')


    # Python 3.x has changed iter.next() to be next(iter) instead, so for
    # backwards compatibility, we'll just define a next() function under 2.x
    def next(iter):
        return iter.next()

# ------------------------------------------------
# Logging is controlled by "python_patch" logger

debugmode = False

logger = logging.getLogger("python_patch")
loghandler = logging.StreamHandler()
logger.addHandler(loghandler)

debug = logger.debug
info = logger.info
warning = logger.warning

# If called as a library, don't log info/debug messages by default.
logger.setLevel(logging.WARN)

# ------------------------------------------------

# constants for patch types

DIFF = PLAIN = "plain"
HG = MERCURIAL = "mercurial"
SVN = SUBVERSION = "svn"


def fromfile(filename):
    """ Parse patch file and return Patch() object
    """
    info("reading patch from file %s" % filename)
    fp = open(filename, "r")
    patch = Patch(fp)
    fp.close()
    return patch


def fromstring(s):
    """ Parse text string and return Patch() object
    """
    return Patch(StringIO(s))


class HunkInfo(object):
    """ Parsed hunk data container (hunk starts with @@ -R +R @@) """

    def __init__(self):
        self.startsrc = None  #: line count starts with 1
        self.linessrc = None
        self.starttgt = None
        self.linestgt = None
        self.invalid = False
        self.text = []

    def copy(self):
        return copy.copy(self)


#  def apply(self, estream):
#    """ write hunk data into enumerable stream
#        return strings one by one until hunk is
#        over
#
#        enumerable stream are tuples (lineno, line)
#        where lineno starts with 0
#    """
#    pass


class Patch(object):

    def __init__(self, stream=None):

        # define Patch data members
        # table with a row for every source file

        #: list of source filenames
        self.source = None
        self.target = None
        #: list of lists of hunks
        self.hunks = None
        #: file endings statistics for every hunk
        self.hunkends = None
        #: headers for each file
        self.header = None

        #: patch type - one of constants
        self.type = None

        if stream:
            self.parse(stream)

    def copy(self):
        return copy.copy(self)

    def parse(self, stream):
        """ parse unified diff """
        self.header = []

        self.source = []
        self.target = []
        self.hunks = []
        self.hunkends = []

        # define possible file regions that will direct the parser flow
        headscan = False  # scanning header before the patch body
        filenames = False  # lines starting with --- and +++

        hunkhead = False  # @@ -R +R @@ sequence
        hunkbody = False  #
        hunkskip = False  # skipping invalid hunk mode

        headscan = True
        lineends = dict(lf=0, crlf=0, cr=0)
        nextfileno = 0
        nexthunkno = 0  #: even if index starts with 0 user messages number hunks from 1

        # hunkinfo holds parsed values, hunkactual - calculated
        hunkinfo = HunkInfo()
        hunkactual = dict(linessrc=None, linestgt=None)

        fe = enumerate(stream)
        for lineno, line in fe:

            # read out header
            if headscan:
                header = ''
                try:
                    while not line.startswith("--- "):
                        header += line
                        lineno, line = next(fe)
                except StopIteration:
                    # this is actually a loop exit
                    continue
                self.header.append(header)

                headscan = False
                # switch to filenames state
                filenames = True

            # hunkskip and hunkbody code skipped until definition of hunkhead is parsed
            if hunkbody:
                # process line first
                if re.match(r"^[- \+\\]", line):
                    # gather stats about line endings
                    if line.endswith("\r\n"):
                        self.hunkends[nextfileno - 1]["crlf"] += 1
                    elif line.endswith("\n"):
                        self.hunkends[nextfileno - 1]["lf"] += 1
                    elif line.endswith("\r"):
                        self.hunkends[nextfileno - 1]["cr"] += 1

                    if line.startswith("-"):
                        hunkactual["linessrc"] += 1
                    elif line.startswith("+"):
                        hunkactual["linestgt"] += 1
                    elif not line.startswith("\\"):
                        hunkactual["linessrc"] += 1
                        hunkactual["linestgt"] += 1
                    hunkinfo.text.append(line)
                    # todo: handle \ No newline cases
                else:
                    warning("invalid hunk no.%d at %d for target file %s" % (
                    nexthunkno, lineno + 1, self.target[nextfileno - 1]))
                    # add hunk status node
                    self.hunks[nextfileno - 1].append(hunkinfo.copy())
                    self.hunks[nextfileno - 1][nexthunkno - 1]["invalid"] = True
                    # switch to hunkskip state
                    hunkbody = False
                    hunkskip = True

                # check exit conditions
                if hunkactual["linessrc"] > hunkinfo.linessrc or hunkactual["linestgt"] > hunkinfo.linestgt:
                    warning("extra hunk no.%d lines at %d for target %s" % (
                    nexthunkno, lineno + 1, self.target[nextfileno - 1]))
                    # add hunk status node
                    self.hunks[nextfileno - 1].append(hunkinfo.copy())
                    self.hunks[nextfileno - 1][nexthunkno - 1]["invalid"] = True
                    # switch to hunkskip state
                    hunkbody = False
                    hunkskip = True
                elif hunkinfo.linessrc == hunkactual["linessrc"] and hunkinfo.linestgt == hunkactual[
                    "linestgt"]:
                    self.hunks[nextfileno - 1].append(hunkinfo.copy())
                    # switch to hunkskip state
                    hunkbody = False
                    hunkskip = True

                    # detect mixed window/unix line ends
                    ends = self.hunkends[nextfileno - 1]
                    if ((ends["cr"] != 0) + (ends["crlf"] != 0) + (ends["lf"] != 0)) > 1:
                        warning("inconsistent line ends in patch hunks for %s" % self.source[nextfileno - 1])
                    if debugmode:
                        debuglines = dict(ends)
                        debuglines.update(file=self.target[nextfileno - 1], hunk=nexthunkno)
                        debug(
                            "crlf: %(crlf)d  lf: %(lf)d  cr: %(cr)d\t - file: %(file)s hunk: %(hunk)d" % debuglines)

            if hunkskip:
                match = re.match("^@@ -(\d+)(,(\d+))? \+(\d+)(,(\d+))?", line)
                if match:
                    # switch to hunkhead state
                    hunkskip = False
                    hunkhead = True
                elif line.startswith("--- "):
                    # switch to filenames state
                    hunkskip = False
                    filenames = True
                    if debugmode and len(self.source) > 0:
                        debug("- %2d hunks for %s" % (
                        len(self.hunks[nextfileno - 1]), self.source[nextfileno - 1]))

            if filenames:
                if line.startswith("--- "):
                    if nextfileno in self.source:
                        warning("skipping invalid patch for %s" % self.source[nextfileno])
                        del self.source[nextfileno]
                        # double source filename line is encountered
                        # attempt to restart from this second line
                    re_filename = "^--- ([^\t]+)"
                    match = re.match(re_filename, line)
                    # todo: support spaces in filenames
                    if match:
                        self.source.append(match.group(1).strip())
                    else:
                        warning("skipping invalid filename at line %d" % lineno)
                        # switch back to headscan state
                        filenames = False
                        headscan = True
                elif not line.startswith("+++ "):
                    if nextfileno in self.source:
                        warning("skipping invalid patch with no target for %s" % self.source[nextfileno])
                        del self.source[nextfileno]
                    else:
                        # this should be unreachable
                        warning("skipping invalid target patch")
                    filenames = False
                    headscan = True
                else:
                    if nextfileno in self.target:
                        warning("skipping invalid patch - double target at line %d" % lineno)
                        del self.source[nextfileno]
                        del self.target[nextfileno]
                        nextfileno -= 1
                        # double target filename line is encountered
                        # switch back to headscan state
                        filenames = False
                        headscan = True
                    else:
                        re_filename = "^\+\+\+ ([^\t]+)"
                        match = re.match(re_filename, line)
                        if not match:
                            warning("skipping invalid patch - no target filename at line %d" % lineno)
                            # switch back to headscan state
                            filenames = False
                            headscan = True
                        else:
                            self.target.append(match.group(1).strip())
                            nextfileno += 1
                            # switch to hunkhead state
                            filenames = False
                            hunkhead = True
                            nexthunkno = 0
                            self.hunks.append([])
                            self.hunkends.append(lineends.copy())
                            continue

            if hunkhead:
                match = re.match("^@@ -(\d+)(,(\d+))? \+(\d+)(,(\d+))?", line)
                if not match:
                    if nextfileno - 1 not in self.hunks:
                        warning(
                            "skipping invalid patch with no hunks for file %s" % self.target[nextfileno - 1])
                        # switch to headscan state
                        hunkhead = False
                        headscan = True
                        continue
                    else:
                        # switch to headscan state
                        hunkhead = False
                        headscan = True
                else:
                    hunkinfo.startsrc = int(match.group(1))
                    hunkinfo.linessrc = 1
                    if match.group(3): hunkinfo.linessrc = int(match.group(3))
                    hunkinfo.starttgt = int(match.group(4))
                    hunkinfo.linestgt = 1
                    if match.group(6): hunkinfo.linestgt = int(match.group(6))
                    hunkinfo.invalid = False
                    hunkinfo.text = []

                    hunkactual["linessrc"] = hunkactual["linestgt"] = 0

                    # switch to hunkbody state
                    hunkhead = False
                    hunkbody = True
                    nexthunkno += 1
                    continue

        if not hunkskip:
            warning("patch file incomplete - %s" % filename)
            # sys.exit(?)
        else:
            # duplicated message when an eof is reached
            if debugmode and len(self.source) > 0:
                debug("- %2d hunks for %s" % (len(self.hunks[nextfileno - 1]), self.source[nextfileno - 1]))

        info("total files: %d  total hunks: %d" % (len(self.source), sum([len(hset) for hset in self.hunks])))

    def apply(self):
        """ apply parsed patch """

        total = len(self.source)
        for fileno, filename in enumerate(self.source):

            f2patch = filename
            if not exists(f2patch):
                f2patch = self.target[fileno]
                if not exists(f2patch):
                    warning("source/target file does not exist\n--- %s\n+++ %s" % (filename, f2patch))
                    continue
            if not isfile(f2patch):
                warning("not a file - %s" % f2patch)
                continue
            filename = f2patch

            info("processing %d/%d:\t %s" % (fileno + 1, total, filename))

            # validate before patching
            f2fp = open(filename)
            hunkno = 0
            hunk = self.hunks[fileno][hunkno]
            hunkfind = []
            hunkreplace = []
            validhunks = 0
            canpatch = False
            for lineno, line in enumerate(f2fp):
                if lineno + 1 < hunk.startsrc:
                    continue
                elif lineno + 1 == hunk.startsrc:
                    hunkfind = [x[1:].rstrip("\r\n") for x in hunk.text if x[0] in " -"]
                    hunkreplace = [x[1:].rstrip("\r\n") for x in hunk.text if x[0] in " +"]
                    # pprint(hunkreplace)
                    hunklineno = 0

                    # todo \ No newline at end of file

                # check hunks in source file
                if lineno + 1 < hunk.startsrc + len(hunkfind) - 1:
                    if line.rstrip("\r\n") == hunkfind[hunklineno]:
                        hunklineno += 1
                    else:
                        debug("hunk no.%d doesn't match source file %s" % (hunkno + 1, filename))
                        # file may be already patched, but we will check other hunks anyway
                        hunkno += 1
                        if hunkno < len(self.hunks[fileno]):
                            hunk = self.hunks[fileno][hunkno]
                            continue
                        else:
                            break

                # check if processed line is the last line
                if lineno + 1 == hunk.startsrc + len(hunkfind) - 1:
                    debug("file %s hunk no.%d -- is ready to be patched" % (filename, hunkno + 1))
                    hunkno += 1
                    validhunks += 1
                    if hunkno < len(self.hunks[fileno]):
                        hunk = self.hunks[fileno][hunkno]
                    else:
                        if validhunks == len(self.hunks[fileno]):
                            # patch file
                            canpatch = True
                            break
            else:
                if hunkno < len(self.hunks[fileno]):
                    warning("premature end of source file %s at hunk %d" % (filename, hunkno + 1))

            f2fp.close()

            if validhunks < len(self.hunks[fileno]):
                if self._match_file_hunks(filename, self.hunks[fileno]):
                    warning("already patched  %s" % filename)
                else:
                    warning("source file is different - %s" % filename)
            if canpatch:
                backupname = filename + ".orig"
                if exists(backupname):
                    warning("can't backup original file to %s - aborting" % backupname)
                else:
                    import shutil
                    shutil.move(filename, backupname)
                    if self.write_hunks(backupname, filename, self.hunks[fileno]):
                        info("successfully patched %s" % filename)
                        unlink(backupname)
                    else:
                        warning("error patching file %s" % filename)
                        shutil.copy(filename, filename + ".invalid")
                        warning("invalid version is saved to %s" % filename + ".invalid")
                        # todo: proper rejects
                        shutil.move(backupname, filename)

        # todo: check for premature eof

    def can_patch(self, filename):
        """ Check if specified filename can be patched. Returns None if file can
        not be found among source filenames. False if patch can not be applied
        clearly. True otherwise.

        :returns: True, False or None
        """
        idx = self._get_file_idx(filename, source=True)
        if idx == None:
            return None
        return self._match_file_hunks(filename, self.hunks[idx])

    def _match_file_hunks(self, filepath, hunks):
        matched = True
        fp = open(abspath(filepath))

        class NoMatch(Exception):
            pass

        lineno = 1
        line = fp.readline()
        hno = None
        try:
            for hno, h in enumerate(hunks):
                # skip to first line of the hunk
                while lineno < h.starttgt:
                    if not len(line):  # eof
                        debug("check failed - premature eof before hunk: %d" % (hno + 1))
                        raise NoMatch
                    line = fp.readline()
                    lineno += 1
                for hline in h.text:
                    if hline.startswith("-"):
                        continue
                    if not len(line):
                        debug("check failed - premature eof on hunk: %d" % (hno + 1))
                        # todo: \ No newline at the end of file
                        raise NoMatch
                    if line.rstrip("\r\n") != hline[1:].rstrip("\r\n"):
                        debug("file is not patched - failed hunk: %d" % (hno + 1))
                        raise NoMatch
                    line = fp.readline()
                    lineno += 1

        except NoMatch:
            matched = False
            # todo: display failed hunk, i.e. expected/found

        fp.close()
        return matched

    def patch_stream(self, instream, hunks):
        """ Generator that yields stream patched with hunks iterable

            Converts lineends in hunk lines to the best suitable format
            autodetected from input
        """

        # todo: At the moment substituted lineends may not be the same
        #       at the start and at the end of patching. Also issue a
        #       warning/throw about mixed lineends (is it really needed?)

        hunks = iter(hunks)

        srclineno = 1

        lineends = {'\n': 0, '\r\n': 0, '\r': 0}

        def get_line():
            """
            local utility function - return line from source stream
            collecting line end statistics on the way
            """
            line = instream.readline()
            # 'U' mode works only with text files
            if line.endswith("\r\n"):
                lineends["\r\n"] += 1
            elif line.endswith("\n"):
                lineends["\n"] += 1
            elif line.endswith("\r"):
                lineends["\r"] += 1
            return line

        for hno, h in enumerate(hunks):
            debug("hunk %d" % (hno + 1))
            # skip to line just before hunk starts
            while srclineno < h.startsrc:
                yield get_line()
                srclineno += 1

            for hline in h.text:
                # todo: check \ No newline at the end of file
                if hline.startswith("-") or hline.startswith("\\"):
                    get_line()
                    srclineno += 1
                    continue
                else:
                    if not hline.startswith("+"):
                        get_line()
                        srclineno += 1
                    line2write = hline[1:]
                    # detect if line ends are consistent in source file
                    if sum([bool(lineends[x]) for x in lineends]) == 1:
                        newline = [x for x in lineends if lineends[x] != 0][0]
                        yield line2write.rstrip("\r\n") + newline
                    else:  # newlines are mixed
                        yield line2write

        for line in instream:
            yield line

    def write_hunks(self, srcname, tgtname, hunks):
        src = open(srcname, "r")
        tgt = open(tgtname, "w")

        debug("processing target file %s" % tgtname)

        tgt.writelines(self.patch_stream(src, hunks))

        tgt.close()
        src.close()
        return True

    def _get_file_idx(self, filename, source=None):
        """ Detect index of given filename within patch.

            :param filename:
            :param source: search filename among sources (True),
                           targets (False), or both (None)
            :returns: int or None
        """
        filename = abspath(filename)
        if source == True or source == None:
            for i, fnm in enumerate(self.source):
                if filename == abspath(fnm):
                    return i
        if source == False or source == None:
            for i, fnm in enumerate(self.target):
                if filename == abspath(fnm):
                    return i


if __name__ == "__main__":
    from optparse import OptionParser
    from os.path import exists
    import sys

    opt = OptionParser(usage="%prog [options] unipatch-file", version="python-patch %s" % __version__)
    opt.add_option("-d", "--debug", action="store_true", dest="debugmode", help="Print debugging messages")
    opt.add_option("-q", "--quiet", action="store_true", dest="quiet",
                   help="Only print messages on warning/error")
    (options, args) = opt.parse_args()

    if not args:
        opt.print_version()
        opt.print_help()
        sys.exit()
    debugmode = options.debugmode
    patchfile = args[0]
    if not exists(patchfile) or not isfile(patchfile):
        sys.exit("patch file does not exist - %s" % patchfile)

    if debugmode:
        loglevel = logging.DEBUG
        logformat = "%(levelname)8s %(message)s"
    elif options.quiet:
        loglevel = logging.WARN
        logformat = "%(message)s"
    else:
        loglevel = logging.INFO
        logformat = "%(message)s"
    logger.setLevel(loglevel)
    loghandler.setFormatter(logging.Formatter(logformat))

    patch = fromfile(patchfile)
    # pprint(patch)
    patch.apply()

    # todo: document and test line ends handling logic - patch.py detects proper line-endings
    #       for inserted hunks and issues a warning if patched file has incosistent line ends



import unittest

import nose.selector
from nose.config import Config
from nose.plugins.base import Plugin
from nose.plugins.manager import PluginManager


class TestSelectorPlugins(unittest.TestCase):

    def test_rejection(self):
        class EvilSelector(Plugin):
            def wantFile(self, filename, package=None):
                if 'good' in filename:
                    return False
                return None

        c = Config(plugins=PluginManager(plugins=[EvilSelector()]))
        s = nose.selector.Selector(c)
        s2 = nose.selector.Selector(Config())

        assert s.wantFile('test_neutral.py')
        assert s2.wantFile('test_neutral.py')

        assert s.wantFile('test_evil.py')
        assert s2.wantFile('test_evil.py')

        assert not s.wantFile('test_good.py')
        assert s2.wantFile('test_good.py')


if __name__ == '__main__':
    unittest.main()



def test_generator_fails_before_yield():
    a = 1 // 0
    yield lambda: True


def test_generator_fails_during_iteration():
    for i in [1, 2, 3, 0, 5, 6]:
        a = 1 // i
        yield lambda: True


def test_ok():
    pass


class TestBuggyGenerators(object):

    def test_generator_fails_before_yield(self):
        a = 1 // 0
        yield lambda: True

    def test_generator_fails_during_iteration(self):
        for i in [1, 2, 3, 0, 5, 6]:
            a = 1 // i
            yield lambda: True

    def test_ok(self):
        pass



import unittest

from nose.plugins.base import IPluginInterface


class TestPluginInterfaces(unittest.TestCase):

    def test_api_methods_present(self):

        from nose.loader import TestLoader
        from nose.selector import Selector

        exclude = ['loadTestsFromGenerator',
                   'loadTestsFromGeneratorMethod'
                   ]

        selfuncs = [f for f in dir(Selector)
                    if f.startswith('want')]
        loadfuncs = [f for f in dir(TestLoader)
                     if f.startswith('load') and not f in exclude]

        others = ['addDeprecated', 'addError', 'addFailure',
                  'addSkip', 'addSuccess', 'startTest', 'stopTest',
                  'prepareTest', 'begin', 'report'
                  ]

        expect = selfuncs + loadfuncs + others

        pd = dir(IPluginInterface)

        for f in expect:
            assert f in pd, "No %s in IPluginInterface" % f
            assert getattr(IPluginInterface, f).__doc__, \
                "No docs for %f in IPluginInterface" % f

    def test_no_instantiate(self):
        try:
            p = IPluginInterface()
        except TypeError:
            pass
        else:
            assert False, \
                "Should not be able to instantiate IPluginInterface"


if __name__ == '__main__':
    unittest.main()



import os
import unittest

from nose import loader

support = os.path.join(os.path.dirname(__file__), 'support')


class TestIssue065(unittest.TestCase):
    def test_dict_wrapper_instance_not_loaded(self):
        wd = os.path.join(support, 'issue065')
        l = loader.TestLoader()  # workingDir=wd)
        tests = l.loadTestsFromDir(wd)
        tests = list(tests)
        self.assertEqual(len(tests), 1)
        tests = list(tests[0])
        assert not tests, "Tests were loaded from module with no tests"


if __name__ == '__main__':
    unittest.main()



import os
import unittest

from nose.plugins import Plugin
from nose.plugins.manager import ZeroNinePlugin
from nose.plugins.plugintest import PluginTester

here = os.path.abspath(os.path.dirname(__file__))

support = os.path.join(os.path.dirname(os.path.dirname(here)), 'support')


class EmptyPlugin(Plugin):
    pass


class TestEmptyPlugin(PluginTester, unittest.TestCase):
    activate = '--with-empty'
    plugins = [ZeroNinePlugin(EmptyPlugin())]
    suitepath = os.path.join(here, 'empty_plugin.rst')

    def test_empty_zero_nine_does_not_crash(self):
        print
        self.output
        assert "'EmptyPlugin' object has no attribute 'loadTestsFromPath'" \
               not in self.output



import unittest

from nose.plugins import Plugin
from nose.plugins.manager import DefaultPluginManager


class OverridesSkip(Plugin):
    """Plugin to override the built-in Skip"""
    enabled = True
    name = 'skip'
    is_overridden = True


class TestDefaultPluginManager(unittest.TestCase):

    def test_extraplugins_override_builtins(self):
        pm = DefaultPluginManager()
        pm.addPlugins(extraplugins=[OverridesSkip()])
        pm.loadPlugins()
        for plugin in pm.plugins:
            if plugin.name == "skip":
                break
        overridden = getattr(plugin, 'is_overridden', False)
        self.assertTrue(overridden)



# -*- coding: utf-8 -*-
import sys
import unittest
from optparse import OptionParser

from nose.config import Config
from nose.plugins.capture import Capture


class TestCapturePlugin(unittest.TestCase):

    def setUp(self):
        self._stdout = sys.stdout

    def tearDown(self):
        sys.stdout = self._stdout

    def test_enabled_by_default(self):
        c = Capture()
        assert c.enabled

    def test_can_be_disabled(self):
        c = Capture()
        parser = OptionParser()
        c.addOptions(parser)
        options, args = parser.parse_args(['test_can_be_disabled',
                                           '-s'])
        c.configure(options, Config())
        assert not c.enabled

        c = Capture()
        options, args = parser.parse_args(['test_can_be_disabled_long',
                                           '--nocapture'])
        c.configure(options, Config())
        assert not c.enabled

        env = {'NOSE_NOCAPTURE': 1}
        c = Capture()
        parser = OptionParser()
        c.addOptions(parser, env)
        options, args = parser.parse_args(['test_can_be_disabled'])
        c.configure(options, Config())
        assert not c.enabled

        c = Capture()
        parser = OptionParser()
        c.addOptions(parser)

        options, args = parser.parse_args(['test_can_be_disabled'])
        c.configure(options, Config())
        assert c.enabled

    def test_captures_stdout(self):
        c = Capture()
        c.start()
        print("Hello")
        c.end()
        self.assertEqual(c.buffer, "Hello\n")

    def test_captures_nonascii_stdout(self):
        c = Capture()
        c.start()
        print("test 日本")
        c.end()
        self.assertEqual(c.buffer, "test 日本\n")

    def test_format_error(self):
        class Dummy:
            pass

        d = Dummy()
        c = Capture()
        c.start()
        try:
            print("Oh my!")
            raise Exception("boom")
        except:
            err = sys.exc_info()
        formatted = c.formatError(d, err)
        ec, ev, tb = err
        (fec, fev, ftb) = formatted
        # print(fec, fev, ftb)

        self.assertEqual(ec, fec)
        self.assertEqual(tb, ftb)
        assert 'Oh my!' in fev, "Output not found in error message"
        assert 'Oh my!' in d.capturedOutput, "Output not attached to test"

    def test_format_nonascii_error(self):
        class Dummy:
            pass

        d = Dummy()
        c = Capture()
        c.start()
        try:
            print("debug 日本")
            raise AssertionError(u'response does not contain 名')
        except:
            err = sys.exc_info()
        formatted = c.formatError(d, err)

    def test_captured_stdout_has_encoding_attribute(self):
        c = Capture()
        c.start()
        self.assertNotEqual(sys.stdout, sys.__stdout__)
        self.assertTrue(hasattr(sys.stdout, 'encoding'))
        c.end()


if __name__ == '__main__':
    unittest.main()



import unittest

from helpers import iter_compat

from nose.suite import LazySuite


def gen():
    for x in range(0, 10):
        yield TestLazySuite.TC('test')


class TestLazySuite(unittest.TestCase):
    class TC(unittest.TestCase):
        def test(self):
            pass

    def test_basic_iteration(self):
        ls = LazySuite(gen)
        for t in iter_compat(ls):
            assert isinstance(t, unittest.TestCase)


if __name__ == '__main__':
    unittest.main()



def __bootstrap__():
    """ Import the code in ``noname_wrapped.not_py`` in file as our own name

    This is a simplified version of the wrapper that setuptools writes for
    dynamic libraries when installing.
    """
    import os
    import imp
    here = os.path.join(os.path.dirname(__file__))
    imp.load_source(__name__, os.path.join(here, 'noname_wrapped.not_py'))


__bootstrap__()



from nose.core import collector, main, run, run_exit, runmodule
from nose.exc import SkipTest, DeprecatedTest
from nose.tools import with_setup

__author__ = 'Jason Pellerin'
__versioninfo__ = (1, 6, 2)
__version__ = '.'.join(map(str, __versioninfo__))

__all__ = [
    'main', 'run', 'run_exit', 'runmodule', 'with_setup',
    'SkipTest', 'DeprecatedTest', 'collector'
    ]



# -*- coding: utf-8 -*-
import codecs
import os
import sys
import unittest

from nose.plugins import PluginTester
from nose.plugins.capture import Capture
from nose.plugins.skip import Skip
from nose.plugins.xunit import Xunit

support = os.path.join(os.path.dirname(__file__), 'support')
xml_results_filename = os.path.join(support, "xunit.xml")


# the plugin is tested better in unit tests.
# this is just here for a sanity check

class TestXUnitPlugin(PluginTester, unittest.TestCase):
    activate = '--with-xunit'
    args = ['-v', '--xunit-file=%s' % xml_results_filename]
    plugins = [Xunit(), Skip()]
    suitepath = os.path.join(support, 'xunit')

    def runTest(self):
        print
        str(self.output)

        assert "ERROR: test_error" in self.output
        assert "FAIL: test_fail" in self.output
        assert "test_skip (test_xunit_as_suite.TestForXunit) ... SKIP: skipit" in self.output
        assert "XML: %s" % xml_results_filename in self.output

        f = codecs.open(xml_results_filename, 'r', encoding='utf8')
        result = f.read()
        f.close()
        print
        result.encode('utf8', 'replace')

        assert '<?xml version="1.0" encoding="UTF-8"?>' in result
        assert '<testsuite name="nosetests" tests="6" errors="2" failures="1" skip="1">' in result
        assert '<testcase classname="test_xunit_as_suite.TestForXunit" name="test_error" time="' in result
        # TODO(Kumar) think of better x-platform code here that
        # does not confuse 2to3
        if sys.version_info[0:2] >= (3, 0):
            assert ('<error type="%s.Exception" message="日本">' % (Exception.__module__,)) in result
        else:
            assert ('<error type="%s.Exception" message="日本">' % (Exception.__module__,)).decode(
                'utf8') in result
        assert '</testcase>' in result
        assert '</testsuite>' in result


class TestIssue134(PluginTester, unittest.TestCase):
    activate = '--with-xunit'
    args = ['-v', '--xunit-file=%s' % xml_results_filename]
    plugins = [Capture(), Xunit()]
    suitepath = os.path.join(support, 'issue134')

    def runTest(self):
        print
        str(self.output)
        f = open(xml_results_filename, 'r')
        result = f.read()
        f.close()
        print
        result
        assert 'raise IOError(42, "test")' in result
        assert 'tests="1" errors="1" failures="0" skip="0"' in result


class TestIssue279(PluginTester, unittest.TestCase):
    activate = '--with-xunit'
    args = ['-v', '--xunit-file=%s' % xml_results_filename]
    plugins = [Xunit(), Skip()]
    suitepath = os.path.join(support, 'issue279')

    def runTest(self):
        print
        str(self.output)
        f = open(xml_results_filename, 'r')
        result = f.read()
        f.close()
        print
        result
        assert 'tests="1" errors="1" failures="0" skip="0"' in result
        assert "Exception: I would prefer not to" in result


class TestIssue680(PluginTester, unittest.TestCase):
    activate = '--with-xunit'
    args = ['-v', '--xunit-file=%s' % xml_results_filename]
    plugins = [Xunit(), Skip()]
    suitepath = os.path.join(support, 'issue680')

    def runTest(self):
        print
        str(self.output)
        f = open(xml_results_filename, 'rb')
        result = f.read().decode('utf-8')
        f.close()
        print
        result
        assert 'tests="1" errors="0" failures="0" skip="0"' in result


class TestIssue700(PluginTester, unittest.TestCase):
    activate = '--with-xunit'
    args = ['-v', '--xunit-file=%s' % xml_results_filename]
    plugins = [Xunit(), Skip()]
    suitepath = os.path.join(support, 'issue700')

    def runTest(self):
        print
        str(self.output)
        f = open(xml_results_filename, 'r')
        result = f.read()
        f.close()
        print
        result
        assert 'tests="1" errors="0" failures="0" skip="0"' in result
        assert 'line1\n' in result
        assert 'line2\n' in result


class TestIssue859(PluginTester, unittest.TestCase):
    activate = '--with-xunit'
    testsuite_name = "TestIssue859"
    args = ['-v', '--xunit-file=%s' % xml_results_filename, '--xunit-testsuite-name=%s' % testsuite_name]
    plugins = [Xunit(), Skip()]
    suitepath = os.path.join(support, 'issue859')

    def runTest(self):
        print
        str(self.output)
        f = open(xml_results_filename, 'r')
        result = f.read()
        f.close()
        print
        result
        assert 'tests="1" errors="0" failures="0" skip="0"' in result
        assert 'testsuite name="TestIssue859"' in result



from state import called


def setup():
    called.append('test_pak2.setup')


def teardown():
    called.append('test_pak2.teardown')


def test_two_one():
    called.append('test_pak2.test_two_one')


def test_two_two():
    called.append('test_pak2.test_two_two')



import unittest


class Something(unittest.TestCase):
    def test_something(self):
        pass


class TestTwo:

    def __repr__(self):
        return 'TestTwo'

    def test_whatever(self):
        pass



print
"*** test_pak imported"
state = []


def setup():
    # print "SETUP CALLED", state, id(state)
    state.append('test_pak.setup')


def teardown():
    # print "TEARDOWN CALLED", state, id(state)
    state.append('test_pak.teardown')



import os
import sys
import unittest

from nose.plugins import PluginTester
from nose.plugins.builtin import Deprecated, Skip
from nose.plugins.skip import SkipTest

support = os.path.join(os.path.dirname(__file__), 'support')


class TestStringException(PluginTester, unittest.TestCase):
    activate = '-v'
    plugins = [Deprecated(), Skip()]
    args = []
    suitepath = os.path.join(support, 'string-exception')

    def test_string_exception_works(self):
        if sys.version_info >= (2, 6):
            raise SkipTest("String exceptions are not supported in this "
                           "version of Python")

        print
        print
        '!' * 70
        print
        str(self.output)
        print
        '!' * 70
        print
        assert 'raise "string exception"' in str(self.output)
        assert 'raise "string exception in setup"' in str(self.output)
        assert 'raise "string exception in teardown"' in str(self.output)
        assert 'raise "string exception in test"' in str(self.output)



"""Test the coverage plugin."""
import os
import shutil
import unittest

from nose.exc import SkipTest
from nose.plugins import PluginTester
from nose.plugins.cover import Coverage

support = os.path.join(os.path.dirname(__file__), 'support')

try:
    import coverage

    # Python 3.3 may accidentally pick up our support area when running the unit
    # tests.  Look for the coverage attribute to make sure we've got the right
    # package.
    hasCoverage = hasattr(coverage, 'coverage')
except ImportError:
    hasCoverage = False


class TestCoveragePlugin(PluginTester, unittest.TestCase):
    activate = "--with-coverage"
    args = ['-v', '--cover-package=blah', '--cover-html', '--cover-min-percentage', '25']
    plugins = [Coverage()]
    suitepath = os.path.join(support, 'coverage')

    def setUp(self):
        if not hasCoverage:
            raise SkipTest('coverage not available; skipping')

        self.cover_file = os.path.join(os.getcwd(), '.coverage')
        self.cover_html_dir = os.path.join(os.getcwd(), 'cover')
        if os.path.exists(self.cover_file):
            os.unlink(self.cover_file)
        if os.path.exists(self.cover_html_dir):
            shutil.rmtree(self.cover_html_dir)
        super(TestCoveragePlugin, self).setUp()

    def runTest(self):
        print(self.output)
        self.assertTrue(
            # Coverage < 4.0
            ("blah        4      3    25%   1" in self.output) or
            # Coverage >= 4.0
            ("blah.py       4      3    25%   1, 5-6" in self.output))
        self.assertTrue("Ran 1 test in" in self.output)
        # Assert coverage html report exists
        self.assertTrue(os.path.exists(os.path.join(self.cover_html_dir,
                                                    'index.html')))
        # Assert coverage data is saved
        self.assertTrue(os.path.exists(self.cover_file))


class TestCoverageMinPercentagePlugin(PluginTester, unittest.TestCase):
    activate = "--with-coverage"
    args = ['-v', '--cover-package=blah', '--cover-min-percentage', '100']
    plugins = [Coverage()]
    suitepath = os.path.join(support, 'coverage')

    def setUp(self):
        if not hasCoverage:
            raise SkipTest('coverage not available; skipping')

        self.cover_file = os.path.join(os.getcwd(), '.coverage')
        self.cover_html_dir = os.path.join(os.getcwd(), 'cover')
        if os.path.exists(self.cover_file):
            os.unlink(self.cover_file)
        if os.path.exists(self.cover_html_dir):
            shutil.rmtree(self.cover_html_dir)
        self.assertRaises(SystemExit,
                          super(TestCoverageMinPercentagePlugin, self).setUp)

    def runTest(self):
        pass


class TestCoverageMinPercentageSinglePackagePlugin(
    PluginTester, unittest.TestCase):
    activate = "--with-coverage"
    args = ['-v', '--cover-package=blah', '--cover-html',
            '--cover-min-percentage', '100']
    plugins = [Coverage()]
    suitepath = os.path.join(support, 'coverage')

    def setUp(self):
        if not hasCoverage:
            raise SkipTest('coverage not available; skipping')

        self.cover_file = os.path.join(os.getcwd(), '.coverage')
        self.cover_html_dir = os.path.join(os.getcwd(), 'cover')
        if os.path.exists(self.cover_file):
            os.unlink(self.cover_file)
        if os.path.exists(self.cover_html_dir):
            shutil.rmtree(self.cover_html_dir)
        self.assertRaises(SystemExit,
                          super(TestCoverageMinPercentageSinglePackagePlugin,
                                self).setUp)

    def runTest(self):
        pass


class TestCoverageMinPercentageSinglePackageWithBranchesPlugin(
    PluginTester, unittest.TestCase):
    activate = "--with-coverage"
    args = ['-v', '--cover-package=blah', '--cover-branches',
            '--cover-html', '--cover-min-percentage', '100']
    plugins = [Coverage()]
    suitepath = os.path.join(support, 'coverage')

    def setUp(self):
        if not hasCoverage:
            raise SkipTest('coverage not available; skipping')

        self.cover_file = os.path.join(os.getcwd(), '.coverage')
        self.cover_html_dir = os.path.join(os.getcwd(), 'cover')
        if os.path.exists(self.cover_file):
            os.unlink(self.cover_file)
        if os.path.exists(self.cover_html_dir):
            shutil.rmtree(self.cover_html_dir)
        self.assertRaises(
            SystemExit,
            super(TestCoverageMinPercentageSinglePackageWithBranchesPlugin,
                  self).setUp)

    def runTest(self):
        pass


class TestCoverageMinPercentageTOTALPlugin(PluginTester, unittest.TestCase):
    activate = "--with-coverage"
    args = ['-v', '--cover-package=blah', '--cover-package=moo',
            '--cover-min-percentage', '100']
    plugins = [Coverage()]
    suitepath = os.path.join(support, 'coverage2')

    def setUp(self):
        if not hasCoverage:
            raise SkipTest('coverage not available; skipping')

        self.cover_file = os.path.join(os.getcwd(), '.coverage')
        self.cover_html_dir = os.path.join(os.getcwd(), 'cover')
        if os.path.exists(self.cover_file):
            os.unlink(self.cover_file)
        if os.path.exists(self.cover_html_dir):
            shutil.rmtree(self.cover_html_dir)
        self.assertRaises(SystemExit,
                          super(TestCoverageMinPercentageTOTALPlugin, self).setUp)

    def runTest(self):
        pass


class TestCoverageMinPercentageWithBranchesTOTALPlugin(
    PluginTester, unittest.TestCase):
    activate = "--with-coverage"
    args = ['-v', '--cover-package=blah', '--cover-package=moo',
            '--cover-branches', '--cover-min-percentage', '100']
    plugins = [Coverage()]
    suitepath = os.path.join(support, 'coverage2')

    def setUp(self):
        if not hasCoverage:
            raise SkipTest('coverage not available; skipping')

        self.cover_file = os.path.join(os.getcwd(), '.coverage')
        self.cover_html_dir = os.path.join(os.getcwd(), 'cover')
        if os.path.exists(self.cover_file):
            os.unlink(self.cover_file)
        if os.path.exists(self.cover_html_dir):
            shutil.rmtree(self.cover_html_dir)
        self.assertRaises(
            SystemExit,
            super(TestCoverageMinPercentageWithBranchesTOTALPlugin, self).setUp)

    def runTest(self):
        pass


if __name__ == '__main__':
    unittest.main()



from setuptools import setup, find_packages

setup(
    name='Some plugin',
    packages=find_packages(),
    entry_points={
        'nose.plugins.0.10': [
            'someplugin = someplugin:SomePlugin'
        ]
    })



import imp
import sys
import unittest

from nose.loader import TestLoader
from nose.plugins import multiprocess
from nose.suite import ContextSuite


class T_fixt:
    def setupClass(cls):
        pass

    setupClass = classmethod(setupClass)

    def test_a(self):
        pass

    def test_b(self):
        pass


class T:
    def test_a(self):
        pass

    def test_b(self):
        pass


class TestMultiProcessTestRunner(unittest.TestCase):

    def test_next_batch_with_classes(self):
        r = multiprocess.MultiProcessTestRunner()
        l = TestLoader()
        tests = list(r.nextBatch(ContextSuite(
            tests=[l.makeTest(T_fixt), l.makeTest(T)])))
        print
        tests
        self.assertEqual(len(tests), 3)

    def test_next_batch_with_module_fixt(self):
        mod_with_fixt = imp.new_module('mod_with_fixt')
        sys.modules['mod_with_fixt'] = mod_with_fixt

        def teardown():
            pass

        class Test(T):
            pass

        mod_with_fixt.Test = Test
        mod_with_fixt.teardown = teardown
        Test.__module__ = 'mod_with_fixt'

        r = multiprocess.MultiProcessTestRunner()
        l = TestLoader()
        tests = list(r.nextBatch(l.loadTestsFromModule(mod_with_fixt)))
        print
        tests
        self.assertEqual(len(tests), 1)

    def test_next_batch_with_module(self):
        mod_no_fixt = imp.new_module('mod_no_fixt')
        sys.modules['mod_no_fixt'] = mod_no_fixt

        class Test2(T):
            pass

        class Test_fixt(T_fixt):
            pass

        mod_no_fixt.Test = Test2
        Test2.__module__ = 'mod_no_fixt'
        mod_no_fixt.Test_fixt = Test_fixt
        Test_fixt.__module__ = 'mod_no_fixt'

        r = multiprocess.MultiProcessTestRunner()
        l = TestLoader()
        tests = list(r.nextBatch(l.loadTestsFromModule(mod_no_fixt)))
        print
        tests
        self.assertEqual(len(tests), 3)

    def test_next_batch_with_generator_method(self):
        class Tg:
            def test_gen(self):
                for i in range(0, 3):
                    yield self.check, i

            def check(self, val):
                pass

        r = multiprocess.MultiProcessTestRunner()
        l = TestLoader()
        tests = list(r.nextBatch(l.makeTest(Tg)))
        print(tests)
        print([r.address(t) for t in tests])
        self.assertEqual(len(tests), 1)

    def test_next_batch_can_split_set(self):

        mod_with_fixt2 = imp.new_module('mod_with_fixt2')
        sys.modules['mod_with_fixt2'] = mod_with_fixt2

        def setup():
            pass

        class Test(T):
            pass

        class Test_fixt(T_fixt):
            pass

        mod_with_fixt2.Test = Test
        mod_with_fixt2.Test_fixt = Test_fixt
        mod_with_fixt2.setup = setup
        mod_with_fixt2._multiprocess_can_split_ = True
        Test.__module__ = 'mod_with_fixt2'
        Test_fixt.__module__ = 'mod_with_fixt2'

        r = multiprocess.MultiProcessTestRunner()
        l = TestLoader()
        tests = list(r.nextBatch(l.loadTestsFromModule(mod_with_fixt2)))
        print(tests)
        self.assertEqual(len(tests), 3)


if __name__ == '__main__':
    unittest.main()



import sys

from setuptools import setup as _setup

py3_args = ['use_2to3', 'convert_2to3_doctests', 'use_2to3_fixers', 'test_dirs', 'test_build_dir',
            'doctest_exts', 'pyversion_patching']

if sys.version_info < (3,):
    # Remove any Python-3.x-only arguments (so they don't generate complaints
    # from 2.x setuptools) and then just pass through to the regular setup
    # routine.
    def setup(*args, **kwargs):
        for a in py3_args:
            if a in kwargs:
                del kwargs[a]
        return _setup(*args, **kwargs)
else:
    import os
    import re
    import logging
    from setuptools import Distribution as _Distribution
    from distutils.core import Command
    from distutils import dir_util, file_util, log
    import setuptools.command.test
    from pkg_resources import normalize_path

    try:
        import patch

        patch.logger.setLevel(logging.WARN)
    except ImportError:
        patch = None

    patchfile_re = re.compile(r'(.*)\.py([0-9.]+)\.patch$')


    def pyversion_patch(filename):
        '''Find the best pyversion-fixup patch for a given filename and apply
           it.
        '''
        dir, file = os.path.split(filename)
        best_ver = (0,)
        patchfile = None
        for dirfile in os.listdir(dir):
            m = patchfile_re.match(dirfile)
            if not m:
                continue
            base, ver = m.groups()
            if base != file:
                continue
            ver = tuple([int(v) for v in ver.split('.')])
            if sys.version_info >= ver and ver > best_ver:
                best_ver = ver
                patchfile = dirfile
        if not patchfile:
            return False
        log.info("Applying %s to %s..." % (patchfile, filename))
        cwd = os.getcwd()
        os.chdir(dir)
        try:
            p = patch.fromfile(patchfile)
            p.apply()
        finally:
            os.chdir(cwd)
        return True


    class Distribution(_Distribution):
        def __init__(self, attrs=None):
            self.test_dirs = []
            self.test_build_dir = None
            self.doctest_exts = ['.py', '.rst']
            self.pyversion_patching = False
            _Distribution.__init__(self, attrs)


    class BuildTestsCommand(Command):
        # Create mirror copy of tests, convert all .py files using 2to3
        user_options = []

        def initialize_options(self):
            self.test_base = None

        def finalize_options(self):
            test_base = self.distribution.test_build_dir
            if not test_base:
                bcmd = self.get_finalized_command('build')
                test_base = bcmd.build_base
            self.test_base = test_base

        def run(self):
            use_2to3 = getattr(self.distribution, 'use_2to3', False)
            test_dirs = getattr(self.distribution, 'test_dirs', [])
            test_base = self.test_base
            bpy_cmd = self.get_finalized_command("build_py")
            lib_base = normalize_path(bpy_cmd.build_lib)
            modified = []
            py_modified = []
            doc_modified = []
            dir_util.mkpath(test_base)
            for testdir in test_dirs:
                for srcdir, dirnames, filenames in os.walk(testdir):
                    destdir = os.path.join(test_base, srcdir)
                    dir_util.mkpath(destdir)
                    for fn in filenames:
                        if fn.startswith("."):
                            # Skip .svn folders and such
                            continue
                        dstfile, copied = file_util.copy_file(
                            os.path.join(srcdir, fn),
                            os.path.join(destdir, fn),
                            update=True)
                        if copied:
                            modified.append(dstfile)
                            if fn.endswith('.py'):
                                py_modified.append(dstfile)
                            for ext in self.distribution.doctest_exts:
                                if fn.endswith(ext):
                                    doc_modified.append(dstfile)
                                    break
            if use_2to3:
                self.run_2to3(py_modified)
                self.run_2to3(doc_modified, True)
            if self.distribution.pyversion_patching:
                if patch is not None:
                    for file in modified:
                        pyversion_patch(file)
                else:
                    log.warn(
                        "Warning: pyversion_patching specified in setup config but patch module not found.  Patching will not be performed.")

            dir_util.mkpath(lib_base)
            self.reinitialize_command('egg_info', egg_base=lib_base)
            self.run_command('egg_info')


    class TestCommand(setuptools.command.test.test):
        # Override 'test' command to make sure 'build_tests' gets run first.
        def run(self):
            self.run_command('build_tests')
            setuptools.command.test.test.run(self)


    def setup(*args, **kwargs):
        kwargs.setdefault('distclass', Distribution)
        cmdclass = kwargs.setdefault('cmdclass', {})
        cmdclass.setdefault('build_tests', BuildTestsCommand)
        cmdclass.setdefault('test', TestCommand)
        return _setup(*args, **kwargs)



try:
    import ez_setup

    ez_setup.use_setuptools()
except ImportError:
    pass

from setuptools import setup

setup(
    name='Example html output plugin',
    version='0.1',
    author='Jason Pellerin',
    author_email='jpellerin+nose@gmail.com',
    description='Example nose html output plugin',
    license='GNU LGPL',
    py_modules=['htmlplug'],
    entry_points={
        'nose.plugins.0.10': [
            'htmlout = htmlplug:HtmlOutput'
        ]
    }

)



import os
import unittest

from io import StringIO

from nose import SkipTest
from nose.config import Config
from nose.core import TestProgram
from nose.plugins.manager import DefaultPluginManager
from nose.result import _TextTestResult

here = os.path.dirname(__file__)
support = os.path.join(here, 'support')


class TestRunner(unittest.TextTestRunner):
    def _makeResult(self):
        self.result = _TextTestResult(
            self.stream, self.descriptions, self.verbosity)
        return self.result

    # Note that all of these tests use a set config to avoid the loading


# of plugins or settings from .noserc.

class TestTestProgram(unittest.TestCase):

    def test_run_support_ctx(self):
        """Collect and run tests in functional_tests/support/ctx

        This should collect no tests in the default configuration, since
        none of the modules have test-like names.
        """
        stream = StringIO()
        runner = TestRunner(stream=stream)
        prog = TestProgram(defaultTest=os.path.join(support, 'ctx'),
                           argv=['test_run_support_ctx'],
                           testRunner=runner,
                           config=Config(),
                           exit=False)
        res = runner.result
        print
        stream.getvalue()
        self.assertEqual(res.testsRun, 0,
                         "Expected to run 0 tests, ran %s" % res.testsRun)
        assert res.wasSuccessful()
        assert not res.errors
        assert not res.failures

    def test_run_support_package2(self):
        """Collect and run tests in functional_tests/support/package2

        This should collect and run 5 tests.
        """
        stream = StringIO()
        runner = TestRunner(stream=stream)
        prog = TestProgram(defaultTest=os.path.join(support, 'package2'),
                           argv=['test_run_support_package2', '-v'],
                           testRunner=runner,
                           config=Config(),
                           exit=False)
        res = runner.result
        print
        stream.getvalue()
        self.assertEqual(res.testsRun, 6,
                         "Expected to run 6 tests, ran %s" % res.testsRun)
        assert res.wasSuccessful()
        assert not res.errors
        assert not res.failures

    def test_run_support_package3(self):
        """Collect and run tests in functional_tests/support/package3

        This should collect and run 2 test. The package layout is:

        lib/
          a.py
        src/
          b.py
        tests/
          test_a.py
          test_b.py
        """
        stream = StringIO()
        runner = TestRunner(stream=stream)

        prog = TestProgram(defaultTest=os.path.join(support, 'package3'),
                           argv=['test_run_support_package3', '-v'],
                           testRunner=runner,
                           config=Config(),
                           exit=False)
        res = runner.result
        print
        stream.getvalue()
        self.assertEqual(res.testsRun, 2,
                         "Expected to run 2 tests, ran %s" % res.testsRun)
        assert res.wasSuccessful()
        assert not res.errors
        assert not res.failures

    def test_run_support_twist(self):
        """Collect and run tests in functional/support/twist

        This should collect and run 4 tests with 2 fails and an error.
        """
        try:
            from twisted.trial.unittest import TestCase
        except ImportError:
            raise SkipTest('twisted not available; skipping')
        stream = StringIO()
        runner = TestRunner(stream=stream, verbosity=2)

        prog = TestProgram(defaultTest=os.path.join(support, 'twist'),
                           argv=['test_run_support_twist'],
                           testRunner=runner,
                           config=Config(stream=stream),
                           exit=False)
        res = runner.result
        print
        stream.getvalue()
        print
        "-----"
        print
        repr(res)

        self.assertEqual(res.testsRun, 4,
                         "Expected to run 4 tests, ran %s" % (res.testsRun,))
        assert not res.wasSuccessful()
        assert len(res.errors) == 1

        # In 12.3, Twisted made their skip functionality match unittests, so the
        # skipped test is no longer reported as a failure.
        import twisted
        v = twisted.version
        if (v.major, v.minor) >= (12, 3):
            assert len(res.failures) == 1
        else:
            assert len(res.failures) == 2

    def test_issue_130(self):
        """Collect and run tests in support/issue130 without error.

        This tests that the result and error classes can handle string
        exceptions.
        """
        import warnings
        warnings.filterwarnings('ignore', category=DeprecationWarning,
                                module='test')

        stream = StringIO()
        runner = TestRunner(stream=stream, verbosity=2)

        prog = TestProgram(defaultTest=os.path.join(support, 'issue130'),
                           argv=['test_issue_130'],
                           testRunner=runner,
                           config=Config(stream=stream,
                                         plugins=DefaultPluginManager()),
                           exit=False)
        res = runner.result
        print
        stream.getvalue()
        self.assertEqual(res.testsRun, 0)  # error is in setup
        assert not res.wasSuccessful()
        assert res.errors
        assert not res.failures

    def test_defaultTest_list(self):
        stream = StringIO()
        runner = TestRunner(stream=stream, verbosity=2)
        tests = [os.path.join(support, 'package2'),
                 os.path.join(support, 'package3')]
        prog = TestProgram(defaultTest=tests,
                           argv=['test_run_support_package2_3', '-v'],
                           testRunner=runner,
                           config=Config(),
                           exit=False)
        res = runner.result
        print
        stream.getvalue()
        self.assertEqual(res.testsRun, 8)

    def test_illegal_packages_not_selected(self):
        stream = StringIO()
        runner = TestRunner(stream=stream, verbosity=2)

        prog = TestProgram(defaultTest=os.path.join(support, 'issue143'),
                           argv=['test_issue_143'],
                           testRunner=runner,
                           config=Config(stream=stream,
                                         plugins=DefaultPluginManager()),
                           exit=False)
        res = runner.result
        print
        stream.getvalue()
        self.assertEqual(res.testsRun, 0)
        assert res.wasSuccessful()
        assert not res.errors
        assert not res.failures


if __name__ == '__main__':
    # import logging
    # logging.basicConfig(level=logging.DEBUG)
    unittest.main()



import optparse
import os
import re
import sys
import unittest

from nose.config import Config
from nose.exc import SkipTest
from nose.plugins.xunit import Xunit, id_split, Tee
from nose.pyversion import UNICODE_STRINGS
from nose.tools import eq_


def mktest():
    class TC(unittest.TestCase):
        def runTest(self):
            pass

    TC.__qualname__ = TC.__name__
    test = TC()
    return test


mktest.__test__ = False

time_taken = re.compile(r'\d\.\d\d')


class TestSplitId(unittest.TestCase):

    def check_id_split(self, cls, name):
        split = id_split('%s.%s' % (cls, name))
        eq_(split[0], cls)
        eq_(split[1], name)

    def test_no_parenthesis(self):
        self.check_id_split("test_parset", "test_args")

    def test_no_dot_in_args(self):
        self.check_id_split("test_parset", "test_args(('x', [1, 2]),)")

    def test_dot_in_args(self):
        self.check_id_split("test_parset", "test_args(('x.y', 1),)")

    def test_grandchild_has_dot_in_args(self):
        self.check_id_split("test_grandparset.test_parset",
                            "test_args(('x.y', 1),)")


class TestOptions(unittest.TestCase):

    def test_defaults(self):
        parser = optparse.OptionParser()
        x = Xunit()
        x.add_options(parser, env={})
        (options, args) = parser.parse_args([])
        eq_(options.xunit_file, "nosetests.xml")

    def test_file_from_environ(self):
        parser = optparse.OptionParser()
        x = Xunit()
        x.add_options(parser, env={'NOSE_XUNIT_FILE': "kangaroo.xml"})
        (options, args) = parser.parse_args([])
        eq_(options.xunit_file, "kangaroo.xml")

    def test_file_from_opt(self):
        parser = optparse.OptionParser()
        x = Xunit()
        x.add_options(parser, env={})
        (options, args) = parser.parse_args(["--xunit-file=blagojevich.xml"])
        eq_(options.xunit_file, "blagojevich.xml")


class TestTee(unittest.TestCase):
    def setUp(self):
        self.orig_stderr = sys.stderr
        sys.stderr = Tee('utf-8', self.orig_stderr)

    def tearDown(self):
        sys.stderr = self.orig_stderr

    def test_tee_has_error_and_encoding_attributes(self):
        tee = Tee('utf-8', sys.stdout)
        self.assertTrue(hasattr(tee, 'encoding'))
        self.assertTrue(hasattr(tee, 'errors'))

    def test_tee_works_with_distutils_log(self):
        try:
            from distutils.log import Log, DEBUG
        except ImportError:
            raise SkipTest("distutils.log not available; skipping")

        l = Log(DEBUG)
        try:
            l.warn('Test')
        except Exception as e:
            self.fail(
                "Exception raised while writing to distutils.log: %s" % (e,))


class TestXMLOutputWithXML(unittest.TestCase):
    def test_prefix_from_environ(self):
        parser = optparse.OptionParser()
        x = Xunit()
        x.add_options(parser, env={'NOSE_XUNIT_PREFIX_WITH_TESTSUITE_NAME': 'true'})
        (options, args) = parser.parse_args([])
        eq_(options.xunit_prefix_class, True)

    def test_prefix_from_opt(self):
        parser = optparse.OptionParser()
        x = Xunit()
        x.add_options(parser, env={})
        (options, args) = parser.parse_args(["--xunit-prefix-with-testsuite-name"])
        eq_(options.xunit_prefix_class, True)


class BaseTestXMLOutputWithXML(unittest.TestCase):
    def configure(self, args):
        parser = optparse.OptionParser()
        self.x.add_options(parser, env={})
        (options, args) = parser.parse_args(args)
        self.x.configure(options, Config())

    def setUp(self):
        self.xmlfile = os.path.abspath(
            os.path.join(os.path.dirname(__file__),
                         'support', 'xunit.xml'))
        self.x = Xunit()

        try:
            import xml.etree.ElementTree
        except ImportError:
            self.ET = False
        else:
            self.ET = xml.etree.ElementTree

    def tearDown(self):
        os.unlink(self.xmlfile)

    def get_xml_report(self):
        class DummyStream:
            pass

        self.x.report(DummyStream())
        f = open(self.xmlfile, 'rb')
        data = f.read()
        f.close()
        return data


class TestXMLOutputWithXMLAndPrefix(BaseTestXMLOutputWithXML):
    def setUp(self):
        super(TestXMLOutputWithXMLAndPrefix, self).setUp()

    def _assert_testcase_classname(self, expected_classname):
        test = mktest()
        self.x.beforeTest(test)
        self.x.addSuccess(test, (None, None, None))

        result = self.get_xml_report()
        print(result)

        if self.ET:
            tree = self.ET.fromstring(result)
            tc = tree.find("testcase")
            eq_(tc.attrib['classname'], expected_classname)
        else:
            # this is a dumb test for 2.4-
            assert ('<testcase classname="%s" name="runTest"' % expected_classname) in result

    def test_addSuccess_default(self):
        self.configure([
            "--with-xunit",
            "--xunit-file=%s" % self.xmlfile,
            "--xunit-prefix-with-testsuite-name"
        ])

        self._assert_testcase_classname('nosetests.test_xunit.TC')

    def test_addSuccess_custom(self):
        custom_testsuite_name = 'eartest'
        self.configure([
            "--with-xunit",
            "--xunit-file=%s" % self.xmlfile,
            "--xunit-testsuite-name=%s" % custom_testsuite_name,
            "--xunit-prefix-with-testsuite-name"
        ])

        self._assert_testcase_classname("%s.test_xunit.TC" % custom_testsuite_name)


class TestXMLOutputWithXML(BaseTestXMLOutputWithXML):
    def setUp(self):
        super(TestXMLOutputWithXML, self).setUp()
        self.configure([
            "--with-xunit",
            "--xunit-file=%s" % self.xmlfile
        ])

    def test_addFailure(self):
        test = mktest()
        self.x.beforeTest(test)
        try:
            raise AssertionError("one is not 'equal' to two")
        except AssertionError:
            some_err = sys.exc_info()

        self.x.addFailure(test, some_err)

        result = self.get_xml_report()
        print(result)

        if self.ET:
            tree = self.ET.fromstring(result)
            eq_(tree.attrib['name'], "nosetests")
            eq_(tree.attrib['tests'], "1")
            eq_(tree.attrib['errors'], "0")
            eq_(tree.attrib['failures'], "1")
            eq_(tree.attrib['skip'], "0")

            tc = tree.find("testcase")
            eq_(tc.attrib['classname'], "test_xunit.TC")
            eq_(tc.attrib['name'], "runTest")
            assert time_taken.match(tc.attrib['time']), (
                    'Expected decimal time: %s' % tc.attrib['time'])

            err = tc.find("failure")
            eq_(err.attrib['type'], "%s.AssertionError" % (AssertionError.__module__,))
            err_lines = err.text.strip().split("\n")
            eq_(err_lines[0], 'Traceback (most recent call last):')
            eq_(err_lines[-1], 'AssertionError: one is not \'equal\' to two')
            eq_(err_lines[-2], '    raise AssertionError("one is not \'equal\' to two")')
        else:
            # this is a dumb test for 2.4-
            assert '<?xml version="1.0" encoding="UTF-8"?>' in result
            assert '<testsuite name="nosetests" tests="1" errors="0" failures="1" skip="0">' in result
            assert '<testcase classname="test_xunit.TC" name="runTest"' in result
            assert '<failure type="exceptions.AssertionError"' in result
            assert "AssertionError: one is not 'equal' to two" in result
            assert "AssertionError(\"one is not 'equal' to two\")" in result
            assert '</failure></testcase></testsuite>' in result

    def test_addFailure_early(self):
        test = mktest()
        try:
            raise AssertionError("one is not equal to two")
        except AssertionError:
            some_err = sys.exc_info()

        # add failure without startTest, due to custom TestResult munging?
        self.x.addFailure(test, some_err)

        result = self.get_xml_report()
        print(result)

        if self.ET:
            tree = self.ET.fromstring(result)
            tc = tree.find("testcase")
            assert time_taken.match(tc.attrib['time']), (
                    'Expected decimal time: %s' % tc.attrib['time'])
        else:
            # this is a dumb test for 2.4-
            assert '<?xml version="1.0" encoding="UTF-8"?>' in result
            assert ('<testcase classname="test_xunit.TC" '
                    'name="runTest" time="0') in result

    def test_addError(self):
        test = mktest()
        self.x.beforeTest(test)
        try:
            raise RuntimeError("some error happened")
        except RuntimeError:
            some_err = sys.exc_info()

        self.x.addError(test, some_err)

        result = self.get_xml_report()
        print(result)

        if self.ET:
            tree = self.ET.fromstring(result)
            eq_(tree.attrib['name'], "nosetests")
            eq_(tree.attrib['tests'], "1")
            eq_(tree.attrib['errors'], "1")
            eq_(tree.attrib['failures'], "0")
            eq_(tree.attrib['skip'], "0")

            tc = tree.find("testcase")
            eq_(tc.attrib['classname'], "test_xunit.TC")
            eq_(tc.attrib['name'], "runTest")
            assert time_taken.match(tc.attrib['time']), (
                    'Expected decimal time: %s' % tc.attrib['time'])

            err = tc.find("error")
            eq_(err.attrib['type'], "%s.RuntimeError" % (RuntimeError.__module__,))
            err_lines = err.text.strip().split("\n")
            eq_(err_lines[0], 'Traceback (most recent call last):')
            eq_(err_lines[-1], 'RuntimeError: some error happened')
            eq_(err_lines[-2], '    raise RuntimeError("some error happened")')
        else:
            # this is a dumb test for 2.4-
            assert '<?xml version="1.0" encoding="UTF-8"?>' in result
            assert '<testsuite name="nosetests" tests="1" errors="1" failures="0" skip="0">' in result
            assert '<testcase classname="test_xunit.TC" name="runTest"' in result
            assert '<error type="exceptions.RuntimeError"' in result
            assert 'RuntimeError: some error happened' in result
            assert '</error></testcase></testsuite>' in result

    def test_non_utf8_error(self):
        # See http://code.google.com/p/python-nose/issues/detail?id=395
        test = mktest()
        self.x.beforeTest(test)
        try:
            raise RuntimeError(chr(128))  # cannot encode as utf8
        except RuntimeError:
            some_err = sys.exc_info()
        self.x.addError(test, some_err)
        result = self.get_xml_report()
        print(repr(result))
        if self.ET:
            tree = self.ET.fromstring(result)
            tc = tree.find("testcase")
            err = tc.find("error")
            if UNICODE_STRINGS:
                eq_(err.attrib['message'],
                    '\x80')
            else:
                eq_(err.attrib['message'],
                    u'\ufffd')
        else:
            # this is a dumb test for 2.4-
            assert 'RuntimeError: \xef\xbf\xbd' in result

    def test_addError_early(self):
        test = mktest()
        try:
            raise RuntimeError("some error happened")
        except RuntimeError:
            some_err = sys.exc_info()

        self.x.startContext(None)

        # call addError without startTest
        # which can happen if setup() raises an error
        self.x.addError(test, some_err)

        result = self.get_xml_report()
        print(result)

        if self.ET:
            tree = self.ET.fromstring(result)
            tc = tree.find("testcase")
            assert time_taken.match(tc.attrib['time']), (
                    'Expected decimal time: %s' % tc.attrib['time'])
        else:
            # this is a dumb test for 2.4-
            assert '<?xml version="1.0" encoding="UTF-8"?>' in result
            assert ('<testcase classname="test_xunit.TC" '
                    'name="runTest" time="0') in result

    def test_addSuccess(self):
        test = mktest()
        self.x.beforeTest(test)
        self.x.addSuccess(test, (None, None, None))

        result = self.get_xml_report()
        print(result)

        if self.ET:
            tree = self.ET.fromstring(result)
            eq_(tree.attrib['name'], "nosetests")
            eq_(tree.attrib['tests'], "1")
            eq_(tree.attrib['errors'], "0")
            eq_(tree.attrib['failures'], "0")
            eq_(tree.attrib['skip'], "0")

            tc = tree.find("testcase")
            eq_(tc.attrib['classname'], "test_xunit.TC")
            eq_(tc.attrib['name'], "runTest")
            assert time_taken.match(tc.attrib['time']), (
                    'Expected decimal time: %s' % tc.attrib['time'])
        else:
            # this is a dumb test for 2.4-
            assert '<?xml version="1.0" encoding="UTF-8"?>' in result
            assert '<testsuite name="nosetests" tests="1" errors="0" failures="0" skip="0">' in result
            assert '<testcase classname="test_xunit.TC" name="runTest"' in result
            assert '</testsuite>' in result

    def test_addSuccess_early(self):
        test = mktest()
        # call addSuccess without startTest
        # which can happen (?) -- did happen with JsLint plugin
        self.x.addSuccess(test, (None, None, None))

        result = self.get_xml_report()
        print(result)

        if self.ET:
            tree = self.ET.fromstring(result)
            tc = tree.find("testcase")
            assert time_taken.match(tc.attrib['time']), (
                    'Expected decimal time: %s' % tc.attrib['time'])
        else:
            # this is a dumb test for 2.4-
            assert '<?xml version="1.0" encoding="UTF-8"?>' in result
            assert ('<testcase classname="test_xunit.TC" '
                    'name="runTest" time="0') in result



import sys

from nose.plugins.skip import SkipTest


def setup():
    if 'java' in sys.version.lower():
        raise SkipTest("StringIO() in jython can't handle unicode")


def test_unicode():
    print(u'b\u00f6y')



from errorclass_failure_plugin import Todo

from nose import SkipTest, DeprecatedTest


def test_todo():
    raise Todo('fix me')


def test_2():
    pass


def test_3():
    raise SkipTest('skipety-skip')


def test_4():
    raise SkipTest()


def test_5():
    raise DeprecatedTest('spam\neggs\n\nspam')


def test_6():
    raise DeprecatedTest('spam')



import os
import unittest

from nose.selector import TestAddress

support = os.path.abspath(os.path.join(os.path.dirname(__file__), 'support'))


class TestTestAddress(unittest.TestCase):

    def test_module_filename(self):
        wd = os.path.join(support, 'package2')
        addr = TestAddress('test_pak.test_mod', workingDir=wd)
        self.assertEqual(addr.filename,
                         os.path.join(wd, 'test_pak', 'test_mod.py'))


if __name__ == '__main__':
    unittest.main()



"""
Tests that plugins can override loadTestsFromTestCase
"""
import os
import unittest

from nose import loader
from nose.plugins import PluginTester
from nose.plugins.base import Plugin

support = os.path.join(os.path.dirname(__file__), 'support')


class NoFixturePlug(Plugin):
    enabled = True

    def options(self, parser, env):
        print
        "options"
        pass

    def configure(self, options, conf):
        print
        "configure"
        pass

    def loadTestsFromTestCase(self, testCaseClass):
        print
        "Called!"

        class Derived(testCaseClass):
            def setUp(self):
                pass

            def tearDown(self):
                pass

        Derived.__qualname__ = Derived.__name__
        # must use nose loader here because the default loader in 2.3
        # won't load tests from base classes
        l = loader.TestLoader()
        return l.loadTestsFromTestCase(Derived)


class TestLoadTestsFromTestCaseHook(PluginTester, unittest.TestCase):
    activate = '-v'
    args = []
    plugins = [NoFixturePlug()]
    suitepath = os.path.join(support, 'ltftc')

    def runTest(self):
        expect = [
            'test_value (%s.Derived) ... ERROR' % __name__,
            'test_value (tests.Tests) ... ok']
        print
        str(self.output)
        for line in self.output:
            if expect:
                self.assertEqual(line.strip(), expect.pop(0))


if __name__ == '__main__':
    unittest.main()
